﻿using System;
using System.Diagnostics;


namespace OfficeIVS
{


	/// <summary>
	/// 文字コードの範囲を表します。
	/// </summary>
	public class CodeRange
	{
		#region フィールド

		/// <summary>
		/// 開始位置。
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private int _startCharCode;

		/// <summary>
		/// 終了位置。
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private int _endCharCode;

		#endregion

		#region コンストラクタ

		/// <summary>
		/// 新しい <see cref="CodeRange"/> 構造体のインスタンスを初期化します。
		/// </summary>
		/// <param name="startCharCode">開始文字コード。</param>
		/// <param name="endCharCode">終了文字コード。</param>
		public CodeRange(int startCharCode, int endCharCode)
		{
			// 初期値値の設定
			SetRange(startCharCode, endCharCode);
		}

		#endregion

		#region プロパティ

		/// <summary>
		/// 範囲の先頭の Unicode コードポイントを取得または設定します。
		/// </summary>
		/// <value>先頭の Unicode コードポイント。</value>
		public int StartCharCode
		{
			[DebuggerStepThrough]
			get { return _startCharCode; }

			[DebuggerStepThrough]
			set { SetRange(value, _endCharCode); }
		}

		/// <summary>
		/// 範囲の末尾の Unicode コードポイントを取得または設定します。
		/// </summary>
		/// <value>末尾の Unicode コードポイント。</value>
		public int EndCharCode
		{
			[DebuggerStepThrough]
			get { return _endCharCode; }

			[DebuggerStepThrough]
			set { SetRange(_startCharCode, value); }
		}

		#endregion

		#region メソッド

		/// <summary>
		/// 指定された範囲を正規化したうえで設定します。
		/// </summary>
		/// <param name="startCharCode">開始文字コード。</param>
		/// <param name="endCharCode">終了文字コード。</param>
		private void SetRange(int startCharCode, int endCharCode)
		{
			if (startCharCode <= endCharCode)
			{
				_startCharCode = startCharCode;
				_endCharCode = endCharCode;
			}
			else
			{
				_startCharCode = endCharCode;
				_endCharCode = startCharCode;
			}
		}

		/// <summary>
		/// 本オブジェクトを表す文字列を取得します。
		/// </summary>
		/// <returns>本オブジェクトを表す文字列。</returns>
		public override string ToString()
		{
			return String.Format("{0:X4}-{1:X4}", _startCharCode, _endCharCode);
		}

		#endregion
	}
}