using System;
using System.Collections.Generic;
using System.IO;
using System.Text;


namespace OfficeIVS.Font.Table
{

	/// <summary>
	/// OpenType フォントの post (PostScript information) テーブルを表します。
	/// </summary>
	[OpenTypeFontTable("post")]
	internal class Post : OpenTypeFontTable
	{
		#region フィールド

		/// <summary>
		/// Apple 標準グリフセットのグリフ名テーブル。
		/// </summary>
		protected static string[] AppleStandardGlyphNames = new[]
		{
			#region グリフ名

			".notdef",				// 0 
			".null",				// 1 
			"nonmarkingreturn",		// 2 
			"space",				// 3 
			"exclam",				// 4 
			"quotedbl",				// 5 
			"numbersign",			// 6 
			"dollar",				// 7 
			"percent",				// 8 
			"ampersand",			// 9 
			"quotesingle",			// 10 
			"parenleft",			// 11 
			"parenright",			// 12 
			"asterisk",				// 13 
			"plus",					// 14 
			"comma",				// 15 
			"hyphen",				// 16 
			"period",				// 17 
			"slash",				// 18 
			"zero",					// 19 
			"one",					// 20 
			"two",					// 21 
			"three",				// 22 
			"four",					// 23 
			"five",					// 24 
			"six",					// 25 
			"seven",				// 26 
			"eight",				// 27 
			"nine",					// 28 
			"colon",				// 29 
			"semicolon",			// 30 
			"less",					// 31 
			"equal",				// 32 
			"greater",				// 33 
			"question",				// 34 
			"at",					// 35 
			"A",					// 36 
			"B",					// 37 
			"C",					// 38 
			"D",					// 39 
			"E",					// 40 
			"F",					// 41 
			"G",					// 42 
			"H",					// 43 
			"I",					// 44 
			"J",					// 45 
			"K",					// 46 
			"L",					// 47 
			"M",					// 48 
			"N",					// 49 
			"O",					// 50 
			"P",					// 51 
			"Q",					// 52 
			"R",					// 53 
			"S",					// 54 
			"T",					// 55 
			"U",					// 56 
			"V",					// 57 
			"W",					// 58 
			"X",					// 59 
			"Y",					// 60 
			"Z",					// 61 
			"bracketleft",			// 62 
			"backslash",			// 63 
			"bracketright",			// 64 
			"asciicircum",			// 65 
			"underscore",			// 66 
			"grave",				// 67 
			"a",					// 68 
			"b",					// 69 
			"c",					// 70 
			"d",					// 71 
			"e",					// 72 
			"f",					// 73 
			"g",					// 74 
			"h",					// 75 
			"i",					// 76 
			"j",					// 77 
			"k",					// 78 
			"l",					// 79 
			"m",					// 80 
			"n",					// 81 
			"o",					// 82 
			"p",					// 83 
			"q",					// 84 
			"r",					// 85 
			"s",					// 86 
			"t",					// 87 
			"u",					// 88 
			"v",					// 89 
			"w",					// 90 
			"x",					// 91 
			"y",					// 92 
			"z",					// 93 
			"braceleft",			// 94 
			"bar",					// 95 
			"braceright",			// 96 
			"asciitilde",			// 97 
			"Adieresis",			// 98 
			"Aring",				// 99 
			"Ccedilla",				// 100 
			"Eacute",				// 101 
			"Ntilde",				// 102 
			"Odieresis",			// 103 
			"Udieresis",			// 104 
			"aacute",				// 105 
			"agrave",				// 106 
			"acircumflex",			// 107 
			"adieresis",			// 108 
			"atilde",				// 109 
			"aring",				// 110 
			"ccedilla",				// 111 
			"eacute",				// 112 
			"egrave",				// 113 
			"ecircumflex",			// 114 
			"edieresis",			// 115 
			"iacute",				// 116 
			"igrave",				// 117 
			"icircumflex",			// 118 
			"idieresis",			// 119 
			"ntilde",				// 120 
			"oacute",				// 121 
			"ograve",				// 122 
			"ocircumflex",			// 123 
			"odieresis",			// 124 
			"otilde",				// 125 
			"uacute",				// 126 
			"ugrave",				// 127 
			"ucircumflex",			// 128 
			"udieresis",			// 129 
			"dagger",				// 130 
			"degree",				// 131 
			"cent",					// 132 
			"sterling",				// 133 
			"section",				// 134 
			"bullet",				// 135 
			"paragraph",			// 136 
			"germandbls",			// 137 
			"registered",			// 138 
			"copyright",			// 139 
			"trademark",			// 140 
			"acute",				// 141 
			"dieresis",				// 142 
			"notequal",				// 143 
			"AE",					// 144 
			"Oslash",				// 145 
			"infinity",				// 146 
			"plusminus",			// 147 
			"lessequal",			// 148 
			"greaterequal",			// 149 
			"yen",					// 150 
			"mu",					// 151 
			"partialdiff",			// 152 
			"summation",			// 153 
			"product",				// 154 
			"pi",					// 155 
			"integral",				// 156 
			"ordfeminine",			// 157 
			"ordmasculine",			// 158 
			"Omega",				// 159 
			"ae",					// 160 
			"oslash",				// 161 
			"questiondown",			// 162 
			"exclamdown",			// 163 
			"logicalnot",			// 164 
			"radical",				// 165 
			"florin",				// 166 
			"approxequal",			// 167 
			"Delta",				// 168 
			"guillemotleft",		// 169 
			"guillemotright",		// 170 
			"ellipsis",				// 171 
			"nonbreakingspace",		// 172 
			"Agrave",				// 173 
			"Atilde",				// 174 
			"Otilde",				// 175 
			"OE",					// 176 
			"oe",					// 177 
			"endash",				// 178 
			"emdash",				// 179 
			"quotedblleft",			// 180 
			"quotedblright",		// 181 
			"quoteleft",			// 182 
			"quoteright",			// 183 
			"divide",				// 184 
			"lozenge",				// 185 
			"ydieresis",			// 186 
			"Ydieresis",			// 187 
			"fraction",				// 188 
			"currency",				// 189 
			"guilsinglleft",		// 190 
			"guilsinglright",		// 191 
			"fi",					// 192 
			"fl",					// 193 
			"daggerdbl",			// 194 
			"periodcentered",		// 195 
			"quotesinglbase",		// 196 
			"quotedblbase",			// 197 
			"perthousand",			// 198 
			"Acircumflex",			// 199 
			"Ecircumflex",			// 200 
			"Aacute",				// 201 
			"Edieresis",			// 202 
			"Egrave",				// 203 
			"Iacute",				// 204 
			"Icircumflex",			// 205 
			"Idieresis",			// 206 
			"Igrave",				// 207 
			"Oacute",				// 208 
			"Ocircumflex",			// 209 
			"apple",				// 210 
			"Ograve",				// 211 
			"Uacute",				// 212 
			"Ucircumflex",			// 213 
			"Ugrave",				// 214 
			"dotlessi",				// 215 
			"circumflex",			// 216 
			"tilde",				// 217 
			"macron",				// 218 
			"breve",				// 219 
			"dotaccent",			// 220 
			"ring",					// 221 
			"cedilla",				// 222 
			"hungarumlaut",			// 223 
			"ogonek",				// 224 
			"caron",				// 225 
			"Lslash",				// 226 
			"lslash",				// 227 
			"Scaron",				// 228 
			"scaron",				// 229 
			"Zcaron",				// 230 
			"zcaron",				// 231 
			"brokenbar",			// 232 
			"Eth",					// 233 
			"eth",					// 234 
			"Yacute",				// 235 
			"yacute",				// 236 
			"Thorn",				// 237 
			"thorn",				// 238 
			"minus",				// 239 
			"multiply",				// 240 
			"onesuperior",			// 241 
			"twosuperior",			// 242 
			"threesuperior",		// 243 
			"onehalf",				// 244 
			"onequarter",			// 245 
			"threequarters",		// 246 
			"franc",				// 247 
			"Gbreve",				// 248 
			"gbreve",				// 249 
			"Idotaccent",			// 250 
			"Scedilla",				// 251 
			"scedilla",				// 252 
			"Cacute",				// 253 
			"cacute",				// 254 
			"Ccaron",				// 255
			"ccaron",				// 256
			"dcroat",				// 257

			#endregion
		};

		#endregion

		#region コンストラクタ

		/// <summary>
		/// 新しい <see cref="Post"/> クラスのインスタンスを初期化します。
		/// </summary>
		/// <param name="version">テーブルバージョン番号。</param>
		public Post(uint version) : base(TableTags.post)
		{
			Version = version;
		}

		#endregion

		#region プロパティ

		/// <summary>
		/// テーブルバージョン番号を取得または設定します。
		/// </summary>
		/// <value>テーブルバージョン番号。</value>
		/// <remarks>
		/// version 1.0 : 0x00010000<br/>
		/// version 2.0 : 0x00020000<br/>
		/// version 2.5 : 0x00025000(deprecated)<br/>
		/// version 3.0 : 0x00030000<br/>
		/// </remarks>
		public uint Version { get; protected set; }

		#endregion

		#region メソッド

		/// <summary>
		/// 指定グリフ番号に対応するグリフ名を取得します。
		/// </summary>
		/// <param name="glyphIndex">フォント内でのグリフ番号。</param>
		/// <returns>グリフ名。</returns>
		/// <exception cref="ArgumentOutOfRangeException"><paramref name="glyphIndex"/> が範囲外。</exception>
		public virtual string GetGlyphName(ushort glyphIndex)
		{
			return String.Empty;
		}

		/// <summary>
		/// フォントから 'post' テーブルを読み込みます。
		/// </summary>
		/// <param name="font">読み込み元となるフォント。</param>
		/// <returns><see cref="T:OfficeIVS.Font.Table.Post"/> 型のインスタンス。テーブルが存在しない場合には <c>null</c>。</returns>
		[OpenTypeFontTableFactoryMethod]
		public static OpenTypeFontTable Load(OpenTypeFont font)
		{
			Post table = null;

			ITableRecord tableRecord = font[TableTags.post];

			if (tableRecord != null)
			{
				using (Stream stream = tableRecord.GetTableStream())
				using (BigEndianBinaryReader reader = new BigEndianBinaryReader(stream))
				{
					uint version = reader.ReadUInt32();

					switch (version)
					{
					case 0x00010000:
						table = new Post10();
						break;

					case 0x00020000:
						table = new Post20();
						break;

					case 0x00025000:
						table = new Post25();
						break;

					case 0x00030000:
						table = new Post30();
						break;

					default:
						table = new Post(version);
						break;
					}

					stream.Seek(-4, SeekOrigin.Current);
					table.LoadEntries(reader, tableRecord.Length);

					table.TableRecord = tableRecord;
				}
			}

			return table;
		}

		/// <summary>
		/// テーブル依存のエントリデータを読み込みます。
		/// </summary>
		/// <param name="input">テーブルデータストリーム。</param>
		/// <param name="length">テーブル長（バイト単位）。</param>
		protected override void LoadEntries(BigEndianBinaryReader input, long length)
		{
			Version = input.ReadUInt32();

			LoadExtendedEntries(input, length - 4);
		}

		/// <summary>
		/// テーブル依存のエントリデータを書き込みます。
		/// </summary>
		/// <param name="output">書き込み先のストリーム。</param>
		protected override void WriteEntries(BigEndianBinaryWriter output)
		{
			output.Write(Version);

			WriteExtendedEntries(output);
		}

		#endregion
	}


	/// <summary>
	/// OpenType フォントの post (PostScript information) テーブル（バージョン 1.0）を表します。
	/// </summary>
	internal class Post10 : Post
	{
		#region コンストラクタ

		/// <summary>
		/// 新しい <see cref="T:OfficeIVS.Font.Table.Post10"/> クラスのインスタンスを初期化します。
		/// </summary>
		public Post10() : this(0x00010000, null)
		{
		}

		/// <summary>
		/// 新しい <see cref="T:OfficeIVS.Font.Table.Post10"/> クラスのインスタンスを初期化します。
		/// </summary>
		/// <param name="post">元となる情報。</param>
		/// <exception cref="NotSupportedException"><paramref name="post"/> は未サポートテーブルバージョン。</exception>
		public Post10(Post post) : this(0x00010000, post)
		{
		}

		/// <summary>
		/// 新しい <see cref="T:OfficeIVS.Font.Table.Post10"/> クラスのインスタンスを初期化します。
		/// </summary>
		/// <param name="version">テーブルバージョン番号。</param>
		/// <param name="post">元となる情報。</param>
		/// <exception cref="NotSupportedException"><paramref name="post"/> は未サポートテーブルバージョン。</exception>
		protected Post10(uint version, Post post) : base(version)
		{
			if (post == null)
			{
				return;
			}

			Post10 source = post as Post10;

			if (source == null)
			{
				throw new NotSupportedException();
			}

			ItalicAngle = source.ItalicAngle;
			UnderlinePosition = source.UnderlinePosition;
			UnderlineThickness = source.UnderlineThickness;
			IsFixedPitch = source.IsFixedPitch;
			MinMemType42 = source.MinMemType42;
			MaxMemType42 = source.MaxMemType42;
			MinMemType1 = source.MinMemType1;
			MaxMemType1 = source.MaxMemType1;
		}

		#endregion

		#region プロパティ

		/// <summary>
		/// 斜体の角度を取得または設定します。
		/// </summary>
		/// <value>斜体の角度（単位度、反時計回りが正）。</value>
		/// <remarks>固定小数点（16.16）形式。</remarks>
		public uint ItalicAngle { get; set; }

		/// <summary>
		/// ベースラインから下線上部までの推奨距離を取得または設定します。
		/// </summary>
		/// <value>ベースラインから下線上部までの推奨距離。</value>
		public short UnderlinePosition { get; set; }

		/// <summary>
		/// 下線の幅の推奨値を取得または設定します。
		/// </summary>
		/// <value>下線の幅の推奨値。</value>
		public short UnderlineThickness { get; set; }

		/// <summary>
		/// 固定ピッチフォントであるかどうかを取得または設定します。
		/// </summary>
		/// <value>固定ピッチフォントの場合には 0 以外、プロポーショナルフォントの場合には 0。</value>
		public uint IsFixedPitch { get; set; }

		/// <summary>
		/// OpenType フォントとしてダウンロードした時に使用する最小メモリ量を取得または設定します。
		/// </summary>
		/// <value>フォントダウンロード時に使用する最小メモリ量。</value>
		/// <remarks>不明な場合には 0。</remarks>
		public uint MinMemType42 { get; set; }

		/// <summary>
		/// OpenType フォントとしてダウンロードした時に使用する最大メモリ量を取得または設定します。
		/// </summary>
		/// <value>フォントダウンロード時に使用する最大メモリ量。</value>
		/// <remarks>不明な場合には 0。</remarks>
		public uint MaxMemType42 { get; set; }

		/// <summary>
		/// Type1 フォントとしてダウンロードした時に使用する最小メモリ量を取得または設定します。
		/// </summary>
		/// <value>フォントダウンロード時に使用する最小メモリ量。</value>
		/// <remarks>不明な場合には 0。</remarks>
		public uint MinMemType1 { get; set; }

		/// <summary>
		/// Type1 フォントとしてダウンロードした時に使用する最大メモリ量を取得または設定します。
		/// </summary>
		/// <value>フォントダウンロード時に使用する最大メモリ量。</value>
		/// <remarks>不明な場合には 0。</remarks>
		public uint MaxMemType1 { get; set; }

		#endregion

		#region メソッド

		/// <summary>
		/// 指定グリフ番号に対応するグリフ名を取得します。
		/// </summary>
		/// <param name="glyphIndex">フォント内でのグリフ番号。</param>
		/// <returns>グリフ名。</returns>
		/// <exception cref="ArgumentOutOfRangeException"><paramref name="glyphIndex"/> が範囲外。</exception>
		public override string GetGlyphName(ushort glyphIndex)
		{
			if (glyphIndex >= AppleStandardGlyphNames.Length)
			{
				throw new ArgumentOutOfRangeException("glyphIndex");
			}

			return AppleStandardGlyphNames[glyphIndex];
		}

		/// <summary>
		/// テーブル依存のエントリデータを読み込みます。
		/// </summary>
		/// <param name="input">テーブルデータストリーム。</param>
		/// <param name="length">テーブル長（バイト単位）。</param>
		protected override void LoadEntries(BigEndianBinaryReader input, long length)
		{
			Version = input.ReadUInt32();
			ItalicAngle = input.ReadUInt32();
			UnderlinePosition = input.ReadInt16();
			UnderlineThickness = input.ReadInt16();
			IsFixedPitch = input.ReadUInt32();
			MinMemType42 = input.ReadUInt32();
			MaxMemType42 = input.ReadUInt32();
			MinMemType1 = input.ReadUInt32();
			MaxMemType1 = input.ReadUInt32();
		}

		/// <summary>
		/// テーブル依存のエントリデータを書き込みます。
		/// </summary>
		/// <param name="output">書き込み先のストリーム。</param>
		protected override void WriteEntries(BigEndianBinaryWriter output)
		{
			output.Write(Version);
			output.Write(ItalicAngle);
			output.Write(UnderlinePosition);
			output.Write(UnderlineThickness);
			output.Write(IsFixedPitch);
			output.Write(MinMemType42);
			output.Write(MaxMemType42);
			output.Write(MinMemType1);
			output.Write(MaxMemType1);
		}

		#endregion
	}


	/// <summary>
	/// OpenType フォントの post (PostScript information) テーブル（バージョン 2.0）を表します。
	/// </summary>
	internal class Post20 : Post10
	{
		#region コンストラクタ

		/// <summary>
		/// 新しい <see cref="T:OfficeIVS.Font.Table.Post20"/> クラスのインスタンスを初期化します。
		/// </summary>
		public Post20() : base(0x00020000, null)
		{
		}

		/// <summary>
		/// 新しい <see cref="T:OfficeIVS.Font.Table.Post20"/> クラスのインスタンスを初期化します。
		/// </summary>
		/// <param name="post">元となる情報。</param>
		/// <exception cref="NotSupportedException"><paramref name="post"/> は未サポートテーブルバージョン。</exception>
		public Post20(Post post) : base(0x00020000, post)
		{
			Post20 source = post as Post20;

			if (source == null)
			{
				throw new NotSupportedException();
			}

			NumberOfGlyphs = source.NumberOfGlyphs;

			GlyphNameIndex = new ushort[NumberOfGlyphs];

			for (int i=0; i<NumberOfGlyphs; i++)
			{
				GlyphNameIndex[i] = source.GlyphNameIndex[i];
			}

			Names = new string[source.Names.Length];

			for (int i=0; i<Names.Length; i++)
			{
				Names[i] = source.Names[i];
			}
		}

		#endregion

		#region プロパティ

		/// <summary>
		/// フォント内のグリフ数を取得または設定します。
		/// </summary>
		/// <value>フォント内のグリフ数。</value>
		/// <remarks>'maxp' テーブルの numGlyphs と同じ値とすべき。</remarks>
		public ushort NumberOfGlyphs { get; set; }

		/// <summary>
		/// グリフ番号とグリフ名の対応コレクションを取得または設定します。
		/// </summary>
		/// <value>グリフ番号とグリフ名の対応。</value>
		/// <remarks>要素数はフォント内のグリフ数。バージョン 2.0 のみ。</remarks>
		public ushort[] GlyphNameIndex { get; set; }

		/// <summary>
		/// グリフ名コレクションを取得または設定します。
		/// </summary>
		/// <value>グリフ名コレクション。</value>
		/// <remarks>Pacal string 型。バージョン 2.0 のみ。</remarks>
		public string[] Names { get; set; }

		#endregion

		#region メソッド

		/// <summary>
		/// 指定グリフ番号に対応するグリフ名を取得します。
		/// </summary>
		/// <param name="glyphIndex">フォント内でのグリフ番号。</param>
		/// <returns>グリフ名。</returns>
		/// <exception cref="ArgumentOutOfRangeException"><paramref name="glyphIndex"/> が範囲外。</exception>
		public override string GetGlyphName(ushort glyphIndex)
		{
			if (glyphIndex >= NumberOfGlyphs)
			{
				throw new ArgumentOutOfRangeException("glyphIndex");
			}

			// glyphNameIndex が 0 から 257 の間は Apple の標準グリフ名を、
			// 258 以上の場合には Names 配列の glyphNameIndex-258 の要素を返す。

			ushort glyphNameIndex = GlyphNameIndex[glyphIndex];

			if (glyphNameIndex <= 257)
			{
				return AppleStandardGlyphNames[glyphIndex];	
			}
	
			return Names[glyphNameIndex - 258];
		}

		/// <summary>
		/// テーブル依存のエントリデータを読み込みます。
		/// </summary>
		/// <param name="input">テーブルデータストリーム。</param>
		/// <param name="length">テーブル長（バイト単位）。</param>
		protected override void LoadEntries(BigEndianBinaryReader input, long length)
		{
			long startPos = input.Position;

			base.LoadEntries(input, length);

			NumberOfGlyphs = input.ReadUInt16();

			// GlyphNameIndex
			GlyphNameIndex = new ushort[NumberOfGlyphs];

			for (int i=0; i<NumberOfGlyphs; i++)
			{
				GlyphNameIndex[i] = input.ReadUInt16();
			}

			long glyphNameTableSize = length - (input.Position - startPos);

			// Names
			List<string> glyphNames = new List<string>();

			byte[] buffer = new byte[256];

			while (glyphNameTableSize > 0)
			{
				int l =	input.ReadByte();

				input.Read(buffer, 0, l);
				glyphNames.Add(Encoding.ASCII.GetString(buffer, 0, l));

				glyphNameTableSize -= l + 1;
			}

			Names = glyphNames.ToArray();
		}

		/// <summary>
		/// テーブル依存のエントリデータを書き込みます。
		/// </summary>
		/// <param name="output">書き込み先のストリーム。</param>
		protected override void WriteEntries(BigEndianBinaryWriter output)
		{
			base.WriteEntries(output);

			output.Write(NumberOfGlyphs);

			foreach (ushort glyphNameIndex in GlyphNameIndex)
			{
				output.Write(glyphNameIndex);
			}

			byte[] buffer = new byte[256];

			foreach (string glyphName in Names)
			{
				// System.String -> PASCAL String
				byte length = Convert.ToByte(Math.Min(glyphName.Length, byte.MaxValue));
				Encoding.ASCII.GetBytes(glyphName, 0, length, buffer, 0);

				output.Write(length);
				output.Write(buffer, 0, length);
			}
		}
	
		#endregion
	}


	/// <summary>
	/// OpenType フォントの post (PostScript information) テーブル（バージョン 2.5）を表します。
	/// </summary>
	internal class Post25 : Post10
	{
		#region コンストラクタ

		/// <summary>
		/// 新しい <see cref="T:OfficeIVS.Font.Table.Post25"/> クラスのインスタンスを初期化します。
		/// </summary>
		public Post25() : base(0x00025000, null)
		{
		}

		/// <summary>
		/// 新しい <see cref="T:OfficeIVS.Font.Table.Post25"/> クラスのインスタンスを初期化します。
		/// </summary>
		/// <param name="post">元となる情報。</param>
		/// <exception cref="NotSupportedException"><paramref name="post"/> は未サポートテーブルバージョン。</exception>
		public Post25(Post post) : base(0x00025000, post)
		{
			Post25 source = post as Post25;

			if (source == null)
			{
				throw new NotSupportedException();
			}

			NumberOfGlyphs = source.NumberOfGlyphs;

			Offset = new sbyte[NumberOfGlyphs];

			for (int i=0; i<NumberOfGlyphs; i++)
			{
				Offset[i] = source.Offset[i];
			}
		}

		#endregion

		#region プロパティ

		/// <summary>
		/// フォント内のグリフ数を取得または設定します。
		/// </summary>
		/// <value>フォント内のグリフ数。</value>
		/// <remarks>'maxp' テーブルの numGlyphs と同じ値とすべき。</remarks>
		public ushort NumberOfGlyphs { get; set; }

		/// <summary>
		/// グリフの Apple 標準の並び位置とのオフセット量を取得または設定します。
		/// </summary>
		/// <value>グリフの Apple 標準の並び位置とのオフセット量のコレクション。</value>
		public sbyte[] Offset { get; set; }

		#endregion

		#region メソッド

		/// <summary>
		/// 指定グリフ番号に対応するグリフ名を取得します。
		/// </summary>
		/// <param name="glyphIndex">フォント内でのグリフ番号。</param>
		/// <returns>グリフ名。</returns>
		/// <exception cref="ArgumentOutOfRangeException"><paramref name="glyphIndex"/> が範囲外。</exception>
		public override string GetGlyphName(ushort glyphIndex)
		{
			if (glyphIndex >= NumberOfGlyphs)
			{
				throw new ArgumentOutOfRangeException("glyphIndex");
			}

			return AppleStandardGlyphNames[glyphIndex + Offset[glyphIndex]];
		}

		/// <summary>
		/// テーブル依存のエントリデータを読み込みます。
		/// </summary>
		/// <param name="input">テーブルデータストリーム。</param>
		/// <param name="length">テーブル長（バイト単位）。</param>
		protected override void LoadEntries(BigEndianBinaryReader input, long length)
		{
			base.LoadEntries(input, length);

			NumberOfGlyphs = input.ReadUInt16();

			Offset = new sbyte[NumberOfGlyphs];

			for (int i=0; i<NumberOfGlyphs; i++)
			{
				Offset[i] = input.ReadInt8();
			}
		}

		/// <summary>
		/// テーブル依存のエントリデータを書き込みます。
		/// </summary>
		/// <param name="output">書き込み先のストリーム。</param>
		protected override void WriteEntries(BigEndianBinaryWriter output)
		{
			base.WriteEntries(output);

			output.Write(NumberOfGlyphs);
			
			foreach (sbyte offset in Offset)
			{
				output.Write(offset);
			}
		}

		#endregion
	}


	/// <summary>
	/// OpenType フォントの post (PostScript information) テーブル（バージョン 3.0）を表します。
	/// </summary>
	internal class Post30 : Post10
	{
		#region コンストラクタ

		/// <summary>
		/// 新しい <see cref="Post30"/> クラスのインスタンスを初期化します。
		/// </summary>
		public Post30() : base(0x00030000, null)
		{
		}

		/// <summary>
		/// 新しい <see cref="T:OfficeIVS.Font.Table.Post30"/> クラスのインスタンスを初期化します。
		/// </summary>
		/// <param name="post">元となる情報。</param>
		/// <exception cref="NotSupportedException"><paramref name="post"/> は未サポートテーブルバージョン。</exception>
		public Post30(Post post) : base(0x00030000, post)
		{
		}

		#endregion

		#region メソッド

		/// <summary>
		/// 指定グリフ番号に対応するグリフ名を取得します。
		/// </summary>
		/// <param name="glyphIndex">フォント内でのグリフ番号。</param>
		/// <returns>グリフ名。</returns>
		/// <exception cref="ArgumentOutOfRangeException"><paramref name="glyphIndex"/> が範囲外。</exception>
		public override string GetGlyphName(ushort glyphIndex)
		{
			// v3.0 はグリフの名前を提供しない
			return String.Empty;
		}

		#endregion
	}
}