﻿using System.Collections.Generic;
using System.Diagnostics;


namespace OfficeIVS.Font
{


	/// <summary>
	/// 定義済みの OpenType テーブルタグ値を提供します。
	/// </summary>
	[DebuggerStepThrough]
	internal static class TableTags
	{
		#region フィールド

		#region  Required Tables

		/// <summary>
		/// cmap
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagCMAP = Tag.FromValue(0x636D6170);

		/// <summary>
		/// head
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagHEAD = Tag.FromValue(0x68656164);

		/// <summary>
		/// hhea
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagHHEA = Tag.FromValue(0x68686561);

		/// <summary>
		/// hmtx
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagHMTX = Tag.FromValue(0x686D7478);

		/// <summary>
		/// maxp
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagMAXP = Tag.FromValue(0x6D617870);

		/// <summary>
		/// name
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagNAME = Tag.FromValue(0x6E616D65);

		/// <summary>
		/// OS2
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagOS2 = Tag.FromValue(0x4F532F32);

		/// <summary>
		/// post
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagPOST = Tag.FromValue(0x706F7374);

		#endregion

		#region Tables Related to TrueType Outlines

		/// <summary>
		/// cvt
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagCVT = Tag.FromValue(0x63767420);

		/// <summary>
		/// fpgm
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagFPGM = Tag.FromValue(0x6670676D);

		/// <summary>
		/// glyf
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagGLYF = Tag.FromValue(0x676C7966);

		/// <summary>
		/// loca
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagLOCA = Tag.FromValue(0x6C6F6361);

		/// <summary>
		/// prep
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagPREP = Tag.FromValue(0x70726570);

		#endregion

		#region Tables Related to PostScript Outlines

		/// <summary>
		/// CFF
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagCFF = Tag.FromValue(0x43464620);

		/// <summary>
		/// VORG
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagVORG = Tag.FromValue(0x564F5247);

		#endregion

		#region Tables Related to Bitmap Glyphs

		/// <summary>
		/// EBDT
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagEBDT = Tag.FromValue(0x45424454);

		/// <summary>
		/// EBLC
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagEBLC = Tag.FromValue(0x45424C43);

		/// <summary>
		/// EBSC
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagEBSC = Tag.FromValue(0x45425343);

		#endregion

		#region Advanced Typographic Tables

		/// <summary>
		/// BASE
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagBASE = Tag.FromValue(0x42415345);

		/// <summary>
		/// GDEF
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagGDEF = Tag.FromValue(0x47444546);

		/// <summary>
		/// GPOS
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagGPOS = Tag.FromValue(0x47504F53);

		/// <summary>
		/// GSUB
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagGSUB = Tag.FromValue(0x47535542);

		/// <summary>
		/// JSTF
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagJSTF = Tag.FromValue(0x4A535446);

		#endregion

		#region Other OpenType Tables

		/// <summary>
		/// DSIG
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagDSIG = Tag.FromValue(0x44534947);

		/// <summary>
		/// gasp
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagGASP = Tag.FromValue(0x67617370);

		/// <summary>
		/// hdmx
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagHDMX = Tag.FromValue(0x68646D78);

		/// <summary>
		/// kern
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagKERN = Tag.FromValue(0x6B65726E);

		/// <summary>
		/// LTSH
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagLTSH = Tag.FromValue(0x4C545348);

		/// <summary>
		/// PCLT
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagPCLT = Tag.FromValue(0x50434C54);

		/// <summary>
		/// VDMX
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagVDMX = Tag.FromValue(0x56444D58);

		/// <summary>
		/// vhea
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagVHEA = Tag.FromValue(0x76686561);

		/// <summary>
		/// vmtx
		/// </summary>
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		private static readonly Tag TagVMTX = Tag.FromValue(0x766D7478);

		#endregion

		#endregion

		#region プロパティ

		#region  Required Tables

		/// <summary>
		/// Character to glyph mapping テーブルタグを取得します。
		/// </summary>
		/// <value>Character to glyph mapping テーブルタグ。</value>
		public static Tag cmap { get { return TagCMAP; } }

		/// <summary>
		/// Font header テーブルタグを取得します。
		/// </summary>
		/// <value>Font header テーブルタグ。</value>
		public static Tag head { get { return TagHEAD; } }

		/// <summary>
		/// Horizontal header テーブルタグを取得します。
		/// </summary>
		/// <value>Horizontal header テーブルタグ。</value>
		public static Tag hhea { get { return TagHHEA; } }

		/// <summary>
		/// Horizontal metrics テーブルタグを取得します。
		/// </summary>
		/// <value>Horizontal metrics テーブルタグ。</value>
		public static Tag hmtx { get { return TagHMTX; } }

		/// <summary>
		/// Maximum profile テーブルタグを取得します。
		/// </summary>
		/// <value>Maximum profile テーブルタグ。</value>
		public static Tag maxp { get { return TagMAXP; } }

		/// <summary>
		/// Naming table テーブルタグを取得します。
		/// </summary>
		/// <value>Naming table テーブルタグ。</value>
		public static Tag name { get { return TagNAME; } }

		/// <summary>
		/// OS/2 and Windows specific metrics テーブルタグを取得します。
		/// </summary>
		/// <value>OS/2 and Windows specific metrics テーブルタグ。</value>
		public static Tag OS2 { get { return TagOS2; } }

		/// <summary>
		/// PostScript information テーブルタグを取得します。
		/// </summary>
		/// <value>PostScript information テーブルタグ。</value>
		public static Tag post { get { return TagPOST; } }

		#endregion

		#region Tables Related to TrueType Outlines

		/// <summary>
		/// Control Value Table テーブルタグを取得します。
		/// </summary>
		/// <value>Control Value Table テーブルタグ。</value>
		public static Tag cvt { get { return TagCVT; } }

		/// <summary>
		/// Font program テーブルタグを取得します。
		/// </summary>
		/// <value>Font program テーブルタグ。</value>
		public static Tag fpgm { get { return TagFPGM; } }

		/// <summary>
		/// Glyph data テーブルタグを取得します。
		/// </summary>
		/// <value>Glyph data テーブルタグ。</value>
		public static Tag glyf { get { return TagGLYF; } }

		/// <summary>
		/// Index to location テーブルタグを取得します。
		/// </summary>
		/// <value>Index to location テーブルタグ。</value>
		public static Tag loca { get { return TagLOCA; } }

		/// <summary>
		/// CVT Program テーブルタグを取得します。
		/// </summary>
		/// <value>CVT Program テーブルタグ。</value>
		public static Tag prep { get { return TagPREP; } }

		#endregion

		#region Tables Related to PostScript Outlines

		/// <summary>
		/// PostScript font program (compact font format) テーブルタグを取得します。
		/// </summary>
		/// <value>PostScript font program (compact font format) テーブルタグ。</value>
		public static Tag CFF { get { return TagCFF; } }

		/// <summary>
		/// Vertical Origin テーブルタグを取得します。
		/// </summary>
		/// <value>Vertical Origin テーブルタグ。</value>
		public static Tag VORG { get { return TagVORG; } }

		#endregion

		#region Tables Related to Bitmap Glyphs

		/// <summary>
		/// Embedded bitmap data テーブルタグを取得します。
		/// </summary>
		/// <value>Embedded bitmap data テーブルタグ。</value>
		public static Tag EBDT { get { return TagEBDT; } }

		/// <summary>
		/// Embedded bitmap location data テーブルタグを取得します。
		/// </summary>
		/// <value>Embedded bitmap location data テーブルタグ。</value>
		public static Tag EBLC { get { return TagEBLC; } }

		/// <summary>
		/// Embedded bitmap scaling data テーブルタグを取得します。
		/// </summary>
		/// <value>Embedded bitmap scaling data テーブルタグ。</value>
		public static Tag EBSC { get { return TagEBSC; } }

		#endregion

		#region Advanced Typographic Tables

		/// <summary>
		/// Baseline data テーブルタグを取得します。
		/// </summary>
		/// <value>Baseline data テーブルタグ。</value>
		public static Tag BASE { get { return TagBASE; } }

		/// <summary>
		/// Glyph definition data テーブルタグを取得します。
		/// </summary>
		/// <value>Glyph definition data テーブルタグ。</value>
		public static Tag GDEF { get { return TagGDEF; } }

		/// <summary>
		/// Glyph positioning data テーブルタグを取得します。
		/// </summary>
		/// <value>Glyph positioning data テーブルタグ。</value>
		public static Tag GPOS { get { return TagGPOS; } }

		/// <summary>
		/// Glyph substitution data テーブルタグを取得します。
		/// </summary>
		/// <value>Glyph substitution data テーブルタグ。</value>
		public static Tag GSUB { get { return TagGSUB; } }

		/// <summary>
		/// Justification data テーブルタグを取得します。
		/// </summary>
		/// <value>Justification data テーブルタグ。</value>
		public static Tag JSTF { get { return TagJSTF; } }

		#endregion

		#region Other OpenType Tables

		/// <summary>
		/// Digital signature テーブルタグを取得します。
		/// </summary>
		/// <value>Digital signature テーブルタグ。</value>
		public static Tag DSIG { get { return TagDSIG; } }

		/// <summary>
		/// Grid-fitting/Scan-conversion テーブルタグを取得します。
		/// </summary>
		/// <value>Grid-fitting/Scan-conversion テーブルタグ。</value>
		public static Tag gasp { get { return TagGASP; } }

		/// <summary>
		/// Horizontal device metrics テーブルタグを取得します。
		/// </summary>
		/// <value>Horizontal device metrics テーブルタグ。</value>
		public static Tag hdmx { get { return TagHDMX; } }

		/// <summary>
		/// Kerning テーブルタグを取得します。
		/// </summary>
		/// <value>Kerning テーブルタグ。</value>
		public static Tag kern { get { return TagKERN; } }

		/// <summary>
		/// Linear threshold data テーブルタグを取得します。
		/// </summary>
		/// <value>Linear threshold data テーブルタグ。</value>
		public static Tag LTSH { get { return TagLTSH; } }

		/// <summary>
		/// PCL 5 data テーブルタグを取得します。
		/// </summary>
		/// <value>PCL 5 data テーブルタグ。</value>
		public static Tag PCLT { get { return TagPCLT; } }

		/// <summary>
		/// Vertical device metrics テーブルタグを取得します。
		/// </summary>
		/// <value>Vertical device metrics テーブルタグ。</value>
		public static Tag VDMX { get { return TagVDMX; } }

		/// <summary>
		/// Vertical Metrics header テーブルタグを取得します。
		/// </summary>
		/// <value>Vertical Metrics header テーブルタグ。</value>
		public static Tag vhea { get { return TagVHEA; } }

		/// <summary>
		/// Vertical Metrics テーブルタグを取得します。
		/// </summary>
		/// <value>Vertical Metrics テーブルタグ。</value>
		public static Tag vmtx { get { return TagVMTX; } }

		#endregion

		#endregion

		#region メソッド

		/// <summary>
		/// OpenType フォントの必須テーブル一覧を取得します。
		/// </summary>
		/// <returns>必須テーブルのタグ一覧。</returns>
		public static IEnumerable<Tag> GetRequiredTableTags()
		{
			return new[] { cmap, head, hhea, hmtx, maxp, name, OS2, post };
		}

		#endregion
	}
}