﻿using System;
using System.Collections.Generic;


namespace OfficeIVS
{

	/// <summary>
	/// 異体字セレクタ付き文字と外字文字とのマッピング情報を提供します。
	/// </summary>
	public interface ICharMap
	{
		#region プロパティ

		/// <summary>
		/// マッピング情報の対象フォント情報を取得します。
		/// </summary>
		/// <value>マッピング情報の対象フォント情報。</value>
		FontInfo FontInfo { get; }

		#endregion

		#region メソッド

		/// <summary>
		/// 予約した外字文字を外字フォントファイルに適用します。
		/// </summary>
		/// <param name="updateLastUsedTime">最終使用日時を更新するかどうか。更新する場合には true（既定値）。</param>
		void ApplyToFont(bool updateLastUsedTime=true);

		/// <summary>
		/// 外字領域の空き領域に指定異体字セレクタ付き文字を予約します。
		/// </summary>
		/// <param name="ivs">異体字セレクタ付き文字。</param>
		/// <returns>予約された外字文字コード。</returns>
		string ReserveEUDC(string ivs);

		/// <summary>
		/// 指定外字文字に関連付けられている異体字セレクタ付き文字を取得します。
		/// </summary>
		/// <param name="eudc">外字文字。</param>
		/// <returns><paramref name="eudc"/> に対応する異体字セレクタ付き文字。関連付けされていない場合は <c>null</c>。</returns>
		string GetIvs(string eudc);

		/// <summary>
		/// 外字領域割り当て済みの異体字セレクタ付き文字の一覧を取得します。
		/// </summary>
		/// <returns>外字領域割り当て済みの異体字セレクタ付き文字のコレクション。</returns>
		List<Tuple<char, string>> GetRegistedIvs();

		#endregion
	}
}