﻿using System;
using System.Threading;


namespace OfficeIVS
{

	/// <summary>
	/// プロセス間でのアクセス同期機能を提供します。
	/// </summary>
	internal class SyncAccess : IDisposable
	{
		#region フィールド

		/// <summary>
		/// アクセス同期用のセマフォオブジェクト。
		/// </summary>
		private Semaphore _semaphore;

		#endregion

		#region コンストラクタ

		/// <summary>
		/// 新しい <see cref="SyncAccess"/> クラスのインスタンスを初期化します。
		/// </summary>
		/// <param name="name">同期名称。</param>
		public SyncAccess(string name)
		{
			_semaphore = new Semaphore(1, 1, name);
			_semaphore.WaitOne();
		}

		#endregion

		#region メソッド

		/// <summary>
		/// 使用しているリソースを解放します。
		/// </summary>
		public void Dispose()
		{
			if (_semaphore == null)
			{
				return;
			}

			_semaphore.Release();
			_semaphore.Dispose();
			_semaphore = null;

			GC.SuppressFinalize(this);
		}

		#endregion
	}
}
