﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;

namespace IvsAddinCommon
{
    abstract public class BaseHelperClass
    {
        protected enum CopyOrCut
        {
            Copy,
            Cut
        }

        private IvsAndEudcCollection mIvsAndEudcList = new IvsAndEudcCollection();

        [System.Diagnostics.DebuggerHidden]
        internal IvsAndEudcCollection IvsAndEudcList
        {
            get
            {
                return this.mIvsAndEudcList;
            }
        }

        /// <summary>
        /// ペースト
        /// </summary>
        /// <param name="officeApp"></param>
        /// <returns></returns>
        public DataObject ClipboardPasteTo(object officeApp, ref bool CancelDefault)
        {
            CancelDefault = false;

            DataObject ret = null;
            RichTextBox rtb = null;

            try
            {
                IDataObject clipboardData = Clipboard.GetDataObject();
                if (clipboardData.GetDataPresent(DataFormats.Rtf) == true)   //Rtfをチェック
                {
                    rtb = new RichTextBox();
                    rtb.Rtf = (string)clipboardData.GetData(DataFormats.Rtf);
                }

                if ((rtb != null) && (0 < rtb.TextLength))
                {
                    bool hiSurrogateContains = rtb.Text.Contains(Utility.IVS_HIGH_SURROGATE_CHAR);
                    if (hiSurrogateContains == true)
                    {
                        Dictionary<string, IvsAndEudc> IvsAndEudcDictionary;
                        bool replaceFlag = this.RichTextReplaceIvs2Eudc(rtb, out IvsAndEudcDictionary);

                        if (replaceFlag == true)
                        {
                            this.PasteRichText(officeApp, clipboardData, rtb, IvsAndEudcDictionary, ref CancelDefault);
                        }
                    }
                }
            }
            catch (ReserveEudcNullResultException ex)
            {
                Utility.ShowErrorMsgBox(ex);
                CancelDefault = false;
            }
            finally
            {
                if (rtb != null)
                {
                    rtb.Dispose();
                }
            }

            return ret;
        }

        public abstract void PasteRichText(object officeApp, IDataObject clipboardData, RichTextBox rtb, Dictionary<string, IvsAndEudc> IvsAndEudcDictionary, ref bool CancelDefault);

        private bool RichTextReplaceIvs2Eudc(RichTextBox rtb, out Dictionary<string, IvsAndEudc> dicIvsAndEudc)
        {
            bool replaceFlag = false;
            dicIvsAndEudc = new Dictionary<string, IvsAndEudc>();

            Application.DoEvents();

            foreach (string item in Utility.IVList)
            {
                if (rtb.Text.Contains(item) == true)
                {
                    for (int i = 0; i < rtb.TextLength; i++)
                    {
                        rtb.Select(i, item.Length);
                        if (rtb.SelectedText == item)
                        {
                            rtb.Select(i - 1, 1);   //親文字を選択
                            string firstCharFontName = rtb.SelectionFont.Name;
                            float firstCharFontSize = rtb.SelectionFont.Size;
                            FontStyle firstCharFontStyle = rtb.SelectionFont.Style;

                            rtb.Select(i - 1, 1 + item.Length);   //異体字を選択
                            string ivs = rtb.SelectedText;

                            OfficeIVS.ICharMap charMap = OfficeIVS.FontManager.GetCharMap(firstCharFontName);
                            string eudc = charMap.ReserveEUDC(rtb.SelectedText);
                            if (eudc == null)
                            {
                                throw new ReserveEudcNullResultException("RichTextReplaceIvs2Eudc");
                            }
                            charMap.ApplyToFont();

                            rtb.SelectedText = eudc;   //差し替え
                            rtb.Select(i - 1, eudc.Length);   //EUDCを選択
                            rtb.SelectionFont = new Font(firstCharFontName, firstCharFontSize, firstCharFontStyle); //フォントの設定
                            replaceFlag = true;

                            IvsAndEudc iae = new IvsAndEudc(ivs, eudc, firstCharFontName);
                            string key = iae.CreateKey();
                            dicIvsAndEudc[key] = iae;
                        }
                    }
                }

                if (rtb.Text.Contains(Utility.IVS_HIGH_SURROGATE_CHAR) == false)
                {
                    break;
                }
            }

            return replaceFlag;
        }

        internal DataObject CreateCopy(IDataObject clipboardData)
        {
            DataObject saveData = new DataObject();
            string[] cbFormatList = clipboardData.GetFormats();
            for (int i = 0; i < cbFormatList.Length; i++)
            {
                object obj = clipboardData.GetData(cbFormatList[i], false);
                saveData.SetData(cbFormatList[i], obj);
            }
            return saveData;
        }

        internal DataObject RichTextEucd2Ivs(ReadOnlyCollection<IvsAndEudc> IvsAndEudcList, RichTextBox rtb, ref bool CancelDefault)
        {
            bool containsFlag = false;
            DataObject dataObj = null;

            for (int i = 0; i < rtb.TextLength; i++)
            {
                rtb.Select(i, 1);
                Debug.WriteLine(i.ToString("#,##0") + "\"" + rtb.SelectedText + "\"");
                foreach (IvsAndEudc item in IvsAndEudcList)
                {
                    if (rtb.SelectedText == item.Eudc)
                    {
                        string firstCharFontName = rtb.SelectionFont.Name;

                        if (firstCharFontName == item.FontName)
                        {
                            OfficeIVS.ICharMap charMap = OfficeIVS.FontManager.GetCharMap(firstCharFontName);
                            string ivs = charMap.GetIvs(rtb.SelectedText);

                            rtb.SelectedText = ivs;   //差し替え

                            rtb.Select(i, ivs.Length);

                            containsFlag = true;
                        }
                    }
                }
            }
            if (containsFlag == true)
            {
                rtb.Select(0, rtb.TextLength);
                dataObj = new DataObject();
                dataObj.SetData(DataFormats.Rtf, true, rtb.SelectedRtf);
                dataObj.SetData(DataFormats.UnicodeText, rtb.Text); //RichTextを受け付けない相手用(入れてあげないとコピーやカットが失敗していると思われるため、文字化けしてても入れておいてあげる)
                Application.DoEvents();
                Clipboard.Clear();
                Clipboard.SetDataObject(dataObj);
                CancelDefault = true;
            }

            return dataObj;
        }
    }
}
