﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Excel = Microsoft.Office.Interop.Excel;
using Office = Microsoft.Office.Core;
using System.Text;
using System.Linq;

namespace IvsAddinCommon
{
    public class ExcelHelper : BaseHelperClass
    {
        private Microsoft.Office.Tools.Excel.ApplicationFactory mAppFactory = null;

        public ExcelHelper()
        {
        }

        public bool HasApplicationFactory
        {
            get
            {
                return this.mAppFactory != null;
            }
        }

        public void SetApplicationFactory(Microsoft.Office.Tools.Excel.ApplicationFactory appFactory)
        {
            if (appFactory != null)
            {
                this.mAppFactory = appFactory;
            }
        }

        public int Ivs2Eudc(Microsoft.Office.Tools.Excel.Workbook Wb)
        {
            return this.Ivs2Eudc(Wb.InnerObject);
        }

        protected int Ivs2Eudc(Excel.Workbook Wb)
        {
            int intResult = 0;

            Wb.Application.Cursor = Excel.XlMousePointer.xlWait;
            Application.DoEvents();
#if !DEBUG
            Wb.Application.ScreenUpdating = false;
#endif
            try
            {
                bool savedFlag = Wb.Saved;
                intResult = this.myIvs2Eudc(Wb);
                Wb.Saved = savedFlag;
            }
            catch (ReserveEudcNullResultException ex)
            {
                Utility.ShowErrorMsgBox(ex);
            }
            finally
            {
                Wb.Application.ScreenUpdating = true;
                Wb.Application.Cursor = Excel.XlMousePointer.xlDefault;
            }
            return intResult;
        }

        private int myIvs2Eudc(Excel.Workbook Wb)
        {
            int counter = 0;
            string[] ivsSelectorList = Utility.IVList;

            base.IvsAndEudcList.Remove(this.mAppFactory.GetVstoObject(Wb));

            for (int i = 0; i < ivsSelectorList.Length; i++)
            {
                string currentIvsSelector = ivsSelectorList[i];
                Excel.Range findRange;

                while (this.DoesExist(Wb, currentIvsSelector, out findRange) == true)
                {
                    string textValue = findRange.Text;

                    int charactersIndex;
                    System.OperatingSystem osInfo = System.Environment.OSVersion;
                    if (osInfo.Platform == PlatformID.Win32NT
                        && (6 == osInfo.Version.Major && 2 <= osInfo.Version.Minor) || (6 < osInfo.Version.Major))
                    {
                        //Win8以降

                        List<byte> ivsSelecterByteList = Encoding.Unicode.GetBytes(currentIvsSelector).ToList();
                        string ivsSelecterByteString = string.Join("", ivsSelecterByteList.ConvertAll(x => "0x" + x.ToString("X2")));

                        int indexCounter = 0;
                        System.Globalization.TextElementEnumerator tee = System.Globalization.StringInfo.GetTextElementEnumerator(textValue);
                        while(tee.MoveNext())
                        {
                            string target = tee.GetTextElement();

                            List<byte> targetByteList = Encoding.Unicode.GetBytes(target).ToList();
                            string targetByteString = string.Join("", targetByteList.ConvertAll(x => "0x" + x.ToString("X2")));

                            if (0 < targetByteString.IndexOf(ivsSelecterByteString))
                            {
                                indexCounter = tee.ElementIndex;
                                break;
                            }
                        }
                        charactersIndex = indexCounter + 1; //VBAなため
                    }
                    else
                    {
                        charactersIndex = textValue.IndexOf(currentIvsSelector) - 1;
                    }

                    Excel.Characters csIvs = findRange.Characters[Start: charactersIndex, Length: currentIvsSelector.Length + 1];
                    string ivsText = csIvs.Text;
                    Excel.Characters csIvsFirstChar = findRange.Characters[Start: charactersIndex, Length: 1];
                    string ivsTextFontName = csIvsFirstChar.Font.Name;

                    OfficeIVS.ICharMap charMap = OfficeIVS.FontManager.GetCharMap(ivsTextFontName);
                    string eudc = charMap.ReserveEUDC(ivsText);
                    if (eudc == null)
                    {
                        throw new ReserveEudcNullResultException("myIvs2EudcmyIvs2Eudc");
                    }
                    charMap.ApplyToFont();
                    counter += this.SearchAndReplace(Wb, ivsText, eudc, ivsTextFontName);

                    Microsoft.Office.Tools.Excel.Workbook myWorkBook = this.mAppFactory.GetVstoObject(Wb);
                    if (base.IvsAndEudcList.Contains(myWorkBook, ivsText, ivsTextFontName) == false)
                    {
                        base.IvsAndEudcList.Add(myWorkBook, ivsText, eudc, ivsTextFontName);
                    }
                    Application.DoEvents();
                }
                if (this.ExistIvs(Wb) == false)
                {
                    break;
                }
            }

            return counter;
        }

        public int Eudc2Ivs(Microsoft.Office.Tools.Excel.Workbook Wb)
        {
            return this.Eudc2Ivs(Wb.InnerObject);
        }

        protected int Eudc2Ivs(Excel.Workbook Wb)
        {
            int intResult = 0;

            Wb.Application.Cursor = Excel.XlMousePointer.xlWait;
            Application.DoEvents();
#if !DEBUG
            Wb.Application.ScreenUpdating = false;
#endif
            try
            {
                bool savedFlag = Wb.Saved;
                intResult = this.myEudc2Ivs(Wb);
                Wb.Saved = savedFlag;
            }
            finally
            {
                Wb.Application.ScreenUpdating = true;
                Wb.Application.Cursor = Excel.XlMousePointer.xlDefault;
            }
            return intResult;
        }

        private int myEudc2Ivs(Excel.Workbook Wb)
        {
            int intResult = 0;

            Microsoft.Office.Tools.Excel.Workbook myWorkBook = this.mAppFactory.GetVstoObject(Wb);
            foreach (IvsAndEudc currentIvsAndEudc in base.IvsAndEudcList[myWorkBook])
            {
                Application.DoEvents();

                this.SearchAndReplace(Wb, currentIvsAndEudc.Eudc, currentIvsAndEudc.Ivs, currentIvsAndEudc.FontName);
            }
            return intResult;
        }

        public bool ExistIvs(Excel.Workbook Wb)
        {
            bool result = false;
            Excel.Range findRange;
            string[] ivsSelectorList = Utility.IVList;

            result = this.DoesExist(Wb, Utility.IVS_HIGH_SURROGATE_CHAR.ToString(), out findRange);
            //for (int i = 0; i < ivsSelectorList.Length; i++)
            //{
            //    result = this.DoesExist(Wb, ivsSelectorList[i], out findRange, false);
            //    if (result == true)
            //    {
            //        break;
            //    }
            //}
            return result;
        }

        protected bool ExistEudc(Excel.Workbook Wb)
        {
            bool ret = true;

            Microsoft.Office.Tools.Excel.Workbook myWorkBook = this.mAppFactory.GetVstoObject(Wb);
            if (0 < base.IvsAndEudcList[myWorkBook].Count)
            {
                ret = true;
            }
            else
            {
                ret = false;
            }
            return ret;
        }

        private bool DoesExist(Excel.Workbook Wb, string target, out Excel.Range findRange)
        {
            bool exist = false;
            findRange = null;

            foreach (Excel.Worksheet ews in Wb.Worksheets)
            {
                findRange = this.FindExecuteFromExist(ews, target);

                if (findRange != null)
                {
                    exist = true;
                    break;
                }
            }
            return exist;
        }

        private int SearchAndReplace(Excel.Workbook wb, string target, string newString, string fontNameFarEast)
        {
            int counter = 0;

            foreach (Excel.Worksheet ews in wb.Worksheets)
            {
                try
                {
                    Excel.Range findRange = this.FindExecute(ews, ews.Cells[1, 1], target, fontNameFarEast);

                    if (findRange != null)
                    {
                        HashSet<string> processedRangeAddressList = new HashSet<string>();
                        processedRangeAddressList.Add(findRange.Address);

                        while (findRange != null)
                        {
                            Debug.WriteLine("Address:" + findRange.Address);

                            counter += this.ReplaceCellChar(findRange, target, newString, fontNameFarEast);

                            Application.DoEvents();

                            findRange = this.FindExecute(ews, findRange, target, fontNameFarEast);  //検索

                            if (findRange != null)
                            {
                                bool result = processedRangeAddressList.Add(findRange.Address);
                                if (result == false)    //重複していればbreak;
                                {
                                    break;
                                }
                            }
                        }
                    }
                }
                finally
                {
                    wb.Application.FindFormat.Clear();  //検索し終えたので検索書式のクリア
                }
            }
            return counter;
        }

        private int ReplaceCellChar(Excel.Range r, string target, string newString, string fontNameFarEast)
        {
            int result = 0;

            if (r != null && r.Characters != null)
            {
                int findIndex = r.Characters.Text.IndexOf(target);

                for (int i = findIndex; i < r.Characters.Count; i++)
                {
                    Excel.Characters chars = r.Characters[Start: i + 1, Length: target.Length];
                    if (chars.Text == target && chars.Font.Name == fontNameFarEast)
                    {
                        chars.Text = newString;
                        result++;

                        //範囲でフォントを再設定
                        chars = r.Characters[Start: i + 1, Length: newString.Length];
                        chars.Font.Name = fontNameFarEast;
                    }

                    if ((i + 1) < r.Characters.Text.Length)
                    {
                        findIndex = r.Characters.Text.IndexOf(target, i + 1);
                        if (findIndex == -1)
                        {
                            break;
                        }
                        else
                        {
                            i = findIndex - 1;  //次のforでi++されるので、-1しておく
                        }
                    }
                }
            }

            return result;
        }

        /// <summary>
        /// Excelの検索メソッドを呼び出す(存在確認する時専用)
        /// </summary>
        /// <param name="ws"></param>
        /// <param name="target"></param>
        /// <param name="lookIn"></param>
        /// <returns></returns>
        private Excel.Range FindExecuteFromExist(Excel.Worksheet ws, string target)
        {
            Excel.Range resultRange = ws.UsedRange.Find(What: target,
                LookIn: Excel.XlFindLookIn.xlValues,
                LookAt: Excel.XlLookAt.xlPart,
                SearchOrder: Excel.XlSearchOrder.xlByRows,
                SearchDirection: Excel.XlSearchDirection.xlNext,
                MatchCase: true,
                /*MatchByte: true,*/
                SearchFormat: false);

            return resultRange;
        }

        /// <summary>
        /// Excelの検索メソッドを呼び出す
        /// </summary>
        /// <param name="ws"></param>
        /// <param name="cellAfter"></param>
        /// <param name="target"></param>
        /// <param name="fontNameFarEast"></param>
        /// <returns></returns>
        private Excel.Range FindExecute(Excel.Worksheet ws, Excel.Range cellAfter, string target, string fontNameFarEast)
        {
            ws.Application.FindFormat.Clear();  //検索書式のクリア
            ws.Application.ReplaceFormat.Clear();   //置換書式のクリア

            Excel.Range resultRange = ws.Cells.Find(What: target,
                After: cellAfter,
                LookIn: Excel.XlFindLookIn.xlFormulas,
                LookAt: Excel.XlLookAt.xlPart,
                SearchOrder: Excel.XlSearchOrder.xlByRows,
                SearchDirection: Excel.XlSearchDirection.xlNext,
                MatchCase: true,
                MatchByte: true,
                SearchFormat: false);

            return resultRange;
        }

        /// <summary>
        /// コピー
        /// </summary>
        /// <param name="app"></param>
        /// <param name="dataObj"></param>
        /// <returns></returns>
        public DataObject ClipboardCopyFromExcel(Excel.Application app, ref bool CancelDefault)
        {
            DataObject dataObj = this.ClipboardCopyOrCut(CopyOrCut.Copy, app, ref CancelDefault);
            return dataObj;
        }

        /// <summary>
        /// カット
        /// </summary>
        /// <param name="app"></param>
        /// <param name="dataObj"></param>
        /// <returns></returns>
        public DataObject ClipboardCutFromExcel(Excel.Application app, ref bool CancelDefault)
        {
            DataObject dataObj = this.ClipboardCopyOrCut(CopyOrCut.Cut, app, ref CancelDefault);
            return dataObj;
        }

        private DataObject ClipboardCopyOrCut(CopyOrCut coc, Excel.Application app, ref bool CancelDefault)
        {
            CancelDefault = false;  //そのままExcelに任せる=false
            DataObject dataObj = null;

            try
            {
                if (app != null && app.Selection != null && app.ActiveWorkbook != null)
                {
                    Microsoft.Office.Tools.Excel.Workbook myWorkbook = this.mAppFactory.GetVstoObject(app.ActiveWorkbook);
                    ReadOnlyCollection<IvsAndEudc> IvsAndEudcList = base.IvsAndEudcList[myWorkbook];
                    if (IvsAndEudcList != null && 0 < IvsAndEudcList.Count)
                    {
                        Application.DoEvents();
                        switch (coc)
                        {
                            case CopyOrCut.Copy:
                                app.Selection.Copy();
                                break;
                            case CopyOrCut.Cut:
                                app.Selection.Cut();
                                break;
                            default:
                                throw new NotImplementedException();
                        }
                        Application.DoEvents();
                        using (RichTextBox rtb = new RichTextBox())
                        {
                            Application.DoEvents();
                            rtb.Paste();
                            Application.DoEvents();

                            dataObj = base.RichTextEucd2Ivs(IvsAndEudcList, rtb, ref CancelDefault);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
                Debug.WriteLine(ex.StackTrace);
                CancelDefault = false;
            }

            return dataObj;
        }


        /// <summary>
        /// ペースト
        /// </summary>
        /// <param name="app"></param>
        /// <param name="target"></param>
        /// <param name="newString"></param>
        /// <param name="dataObj"></param>
        public DataObject ClipboardPasteToExcel(Excel.Application app, ref bool CancelDefault)
        {
            DataObject dataObj = base.ClipboardPasteTo(app, ref CancelDefault);
            return dataObj;
        }

        public void SaveEvent(Microsoft.Office.Tools.Excel.Workbook wb, Office.IRibbonControl control, ref bool CancelDefault)
        {
            CancelDefault = false;
        }

        public void WorkbookOpen(Excel.Workbook Wb)
        {
            Microsoft.Office.Tools.Excel.Workbook myWorkbook = this.mAppFactory.GetVstoObject(Wb);
            base.IvsAndEudcList.Remove(myWorkbook);
            if (this.ExistIvs(Wb) == true)
            {
                this.Ivs2Eudc(Wb);
            }
        }

        /// <summary>
        /// 確定ボタン押下
        /// </summary>
        /// <param name="app"></param>
        /// <param name="eudc"></param>
        /// <param name="fontName"></param>
        /// <param name="caretIndex"></param>
        public void DecisionClicked(Excel.Application app, string eudc, string fontName, int caretIndex)
        {
            object slc = app.Selection;
            Excel.Range range = app.ActiveCell;

            if (range != null)
            {
                app.Cursor = Excel.XlMousePointer.xlWait;
                System.Windows.Forms.Application.DoEvents();
                app.ScreenUpdating = false;
                try
                {
                    int charCount = 0;
                    string value2 = null;

                    // 数値が入っている場合
                    if (range.Value2 != null &&
                        (range.Value2.GetType() == typeof(int) || range.Value2.GetType() == typeof(double)))
                    {
                        value2 = range.Value2.ToString();
                        charCount = caretIndex; //ループの開始位置
                        // 数値のみ入力されているとCharactersが使えないので、Value2に格納する
                        range.Value2 = value2.Insert(charCount, eudc);
                        // 数値のみのセルにキャレットを表示させている場合、以下の処理を入れておくとエラーにならない
                        range.Characters.Text = value2.Insert(charCount, eudc);
                    }
                    // 文字が入っている場合
                    else
                    {
                        charCount = caretIndex; //ループの開始位置
                        range.Characters[Start: charCount + 1, Length: 0].Insert(eudc);
                    }

                    Application.DoEvents();

                    for (int i = charCount + 1; i <= range.Characters.Count; i++)
                    {
                        Excel.Characters chars = range.Characters[Start: i, Length: eudc.Length];
                        if (chars.Text == eudc)
                        {
                            chars.Font.Name = fontName;
                        }
                    }
                    Application.DoEvents();

                    Excel.Workbook interopWorkbook = range.Worksheet.Parent as Excel.Workbook;
                    Microsoft.Office.Tools.Excel.Workbook myWorkbook = this.mAppFactory.GetVstoObject(interopWorkbook);
                    OfficeIVS.ICharMap charMap = OfficeIVS.FontManager.GetCharMap(fontName);
                    string ivs = charMap.GetIvs(eudc);
                    base.IvsAndEudcList.Add(myWorkbook, ivs, eudc, fontName);
                }
                finally
                {
                    //SendKeys.Send("{F2}");
                    app.ScreenUpdating = true;
                    app.Cursor = Excel.XlMousePointer.xlDefault;
                }
            }
        }

        public override void PasteRichText(object officeApp, IDataObject clipboardData, RichTextBox rtb, Dictionary<string, IvsAndEudc> IvsAndEudcDictionary, ref bool CancelDefault)
        {
            Excel.Application appExcel = officeApp as Excel.Application;

            if (appExcel != null && appExcel.Selection != null && appExcel.ActiveWorkbook != null)
            {
                Excel.Range range = appExcel.Selection as Excel.Range;

                if (range != null)
                {
                    DataObject saveData = base.CreateCopy(clipboardData);

                    DataObject ret = new DataObject();
                    rtb.Select(0, rtb.TextLength);
                    Application.DoEvents();
                    ret.SetData(DataFormats.Rtf, true, rtb.SelectedRtf);
                    Application.DoEvents();
                    Clipboard.Clear();
                    Application.DoEvents();
                    Clipboard.SetDataObject(ret);
                    Application.DoEvents();

                    try
                    {
                        range.PasteSpecial(Paste: Excel.XlPasteType.xlPasteAll, Operation: Excel.XlPasteSpecialOperation.xlPasteSpecialOperationNone);
                        CancelDefault = true;
                    }
                    catch (COMException comEx)
                    {
                        if (1 < range.Count)
                        {
                            range = range[1];  //VBAなので1
                        }
                        range.Clear();
                        range.Value = rtb.Text;

                        for (int i = 0; i < rtb.TextLength; i++)
                        {
                            rtb.Select(i, 1);
                            if (1 == rtb.SelectedText.Length)
                            {
                                Font fnt = rtb.SelectionFont;

                                Excel.Characters characters = range.Characters[i + 1, 1];

                                characters.Font.Name = fnt.Name;
                                characters.Font.Size = fnt.Size;
                                characters.Font.Color = System.Drawing.ColorTranslator.ToOle(rtb.SelectionColor);
                                characters.Font.Bold = fnt.Bold;
                                characters.Font.Italic = fnt.Italic;
                                characters.Font.Underline = fnt.Underline;
                                characters.Font.Strikethrough = fnt.Strikeout;
                            }
                        }
                        CancelDefault = true;
                    }
                    finally
                    {
                        Application.DoEvents();
                        Clipboard.SetDataObject(saveData);
                        Application.DoEvents();
                    }

                    if (CancelDefault == true)
                    {
                        foreach (string item in IvsAndEudcDictionary.Keys)
                        {
                            IvsAndEudc iae = IvsAndEudcDictionary[item];
                            Microsoft.Office.Tools.Excel.Workbook myWorkBook = this.mAppFactory.GetVstoObject(appExcel.ActiveWorkbook);
                            this.IvsAndEudcList.Add(myWorkBook, iae.Ivs, iae.Eudc, iae.FontName);
                        }
                    }
                }
            }
        }

        private static System.Timers.Timer aTimer = null;
        private static object timerLockObject = new object();
        private static Excel.Workbook Ivs2EudcWorkbook = null;

        virtual public void WorkbookBeforeSave(Excel.Workbook Wb, bool SaveAsUI, ref bool Cancel)
        {
            if (this.ExistEudc(Wb) == true)
            {
                this.Eudc2Ivs(Wb);

                if (aTimer == null)
                {
                    Ivs2EudcWorkbook = Wb;
                    aTimer = new System.Timers.Timer(1000);
                    aTimer.Elapsed += new System.Timers.ElapsedEventHandler(aTimer_Elapsed);
                    aTimer.Enabled = true;
                }
            }
        }

        void aTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            aTimer.Enabled = false;
            try
            {
                if (aTimer != null)
                {
                    lock (timerLockObject)
                    {
                        if (aTimer != null)
                        {
                            if (this.ExistIvs(Ivs2EudcWorkbook) == true)
                            {
                                this.Ivs2Eudc(Ivs2EudcWorkbook);
                            }
                            aTimer.Enabled = false;
                            aTimer.Dispose();
                            aTimer = null;
                            Ivs2EudcWorkbook = null;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                aTimer.Enabled = true;
                Debug.WriteLine(ex.Message);
                Debug.WriteLine(ex.Source);
                Debug.WriteLine(ex.StackTrace);
                //throw;
            }
        }

        public void WorkbookBeforeClose(Excel.Workbook Wb, ref bool Cancel)
        {
        }

        public List<AssistToolDialog.AssistControl.ExStringItem> Range2ExStringList(Excel.Range range)
        {
            List<AssistToolDialog.AssistControl.ExStringItem> ret = new List<AssistToolDialog.AssistControl.ExStringItem>();
            if (range != null)
            {
                if (range.Value2 != null &&
                    (range.Value2.GetType() == typeof(int) || range.Value2.GetType() == typeof(double)))
                {
                    string valueString = ((double)range.Value2).ToString();

                    for (int i = 0; i < valueString.Length; i++)
                    {
                        ret.Add(new AssistToolDialog.AssistControl.ExStringItem(valueString.Substring(i, 1), range.Font.Name, float.Parse(range.Font.Size.ToString())));
                    }
                }
                else if (0 < range.Characters.Count)
                {
                    Excel.Characters excelChar;
                    for (int i = 1; i <= range.Characters.Count; i++)
                    {
                        // サロゲート文字の判定
                        if (!Char.IsSurrogate(range.Characters.Text[i - 1]))
                        {
                            excelChar = range.get_Characters(Start: i, Length: 1);
                        }
                        else
                        {
                            excelChar = range.get_Characters(Start: i, Length: 2);
                            i++;
                        }

                        ret.Add(new AssistToolDialog.AssistControl.ExStringItem(excelChar.Text, excelChar.Font.Name, float.Parse(excelChar.Font.Size.ToString())));
                    }
                }
            }
            return ret;
        }
    }
}
