﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using Microsoft.Office.Interop.PowerPoint;
using Microsoft.Office.Tools.Excel;
using Microsoft.Office.Tools.Word;

namespace IvsAddinCommon
{
    public class IvsAndEudcCollection
    {
        private Dictionary<Guid, Dictionary<string, IvsAndEudc>> myCollection = new Dictionary<Guid, Dictionary<string, IvsAndEudc>>();

        private void myAdd(Guid id, string ivs, string eudc, string fontName)
        {
            if (id == Guid.Empty)
            {
                throw new ArgumentException("idがGuid.Emptyです");
            }
            if (string.IsNullOrWhiteSpace(ivs))
            {
                throw new ArgumentNullException("ivsがnullまたは空です");
            }
            if (string.IsNullOrWhiteSpace(eudc))
            {
                throw new ArgumentNullException("ivsがnullまたは空です");
            }
            if (string.IsNullOrWhiteSpace(fontName))
            {
                throw new ArgumentNullException("ivsがnullまたは空です");
            }

            Dictionary<string, IvsAndEudc> dicIvsAndEudc;

            if (this.myCollection.ContainsKey(id) == true)
            {
                dicIvsAndEudc = this.myCollection[id];
            }
            else
            {
                dicIvsAndEudc = new Dictionary<string, IvsAndEudc>();
                this.myCollection.Add(id, dicIvsAndEudc);
            }

            IvsAndEudc iae;
            string itemKey = IvsAndEudc.CreateKey(ivs, fontName);
            if (dicIvsAndEudc.Keys.Contains(itemKey) == true)
            {
                iae = dicIvsAndEudc[itemKey];   //重複
            }
            else
            {
                iae = new IvsAndEudc(ivs, eudc, fontName);
                dicIvsAndEudc.Add(itemKey, iae);
            }
        }

        public void Add(Workbook wb, string ivs, string eudc, string fontName)
        {
            this.myAdd(Utility.GetID(wb), ivs, eudc, fontName);
        }

        public void Add(Document doc, string ivs, string eudc, string fontName)
        {
            this.myAdd(Utility.GetID(doc), ivs, eudc, fontName);
        }

        public void Add(Presentation ppt, string ivs, string eudc, string fontName)
        {
            this.myAdd(Utility.GetID(ppt), ivs, eudc, fontName);
        }

        private bool myContains(Guid id, string ivs, string fontName)
        {
            bool ret = false;

            if (this.myCollection.ContainsKey(id) == true)
            {
                Dictionary<string, IvsAndEudc> dicIvsAndEudc = this.myCollection[id];
                string itemKey = IvsAndEudc.CreateKey(ivs, fontName);
                ret = dicIvsAndEudc.Keys.Contains(itemKey);
            }
            return ret;
        }

        public bool Contains(Workbook wb, string ivs, string fontName)
        {
            return this.myContains(Utility.GetID(wb), ivs, fontName);
        }

        public bool Contains(Document doc, string ivs, string fontName)
        {
            return this.myContains(Utility.GetID(doc), ivs, fontName);
        }

        public bool Contains(Presentation ppt, string ivs, string fontName)
        {
            return this.myContains(Utility.GetID(ppt), ivs, fontName);
        }

        private bool myRemove(Guid id)
        {
            return this.myCollection.Remove(id);
        }

        public bool Remove(Workbook wb)
        {
            return this.myRemove(Utility.GetID(wb));
        }

        public bool Remove(Document doc)
        {
            return this.myRemove(Utility.GetID(doc));
        }

        public bool Remove(Presentation ppt)
        {
            return this.myRemove(Utility.GetID(ppt));
        }

        private bool myRemove(Guid id, string ivs, string fontName)
        {
            bool ret = false;

            if (id == Guid.Empty)
            {
                throw new ArgumentException("idがGuid.Emptyです");
            }
            if (string.IsNullOrWhiteSpace(ivs))
            {
                throw new ArgumentNullException("ivsがnullまたは空です");
            }
            if (string.IsNullOrWhiteSpace(fontName))
            {
                throw new ArgumentNullException("ivsがnullまたは空です");
            }

            if (this.myCollection.ContainsKey(id) == true)
            {
                Dictionary<string, IvsAndEudc> dicIvsAndEudc = this.myCollection[id];
                string itemKey = IvsAndEudc.CreateKey(ivs, fontName);
                ret = dicIvsAndEudc.Remove(itemKey);
            }
            return ret;
        }

        public bool Remove(Workbook wb, string ivs, string fontName)
        {
            return this.myRemove(Utility.GetID(wb), ivs, fontName);
        }

        public bool Remove(Document doc, string ivs, string fontName)
        {
            return this.myRemove(Utility.GetID(doc), ivs, fontName);
        }

        public bool Remove(Presentation ppt, string ivs, string fontName)
        {
            return this.myRemove(Utility.GetID(ppt), ivs, fontName);
        }

        private ReadOnlyCollection<IvsAndEudc> GetItem(Guid id)
        {
            ReadOnlyCollection<IvsAndEudc> ret;
            Dictionary<string, IvsAndEudc> find;

            if (this.myCollection.ContainsKey(id) == true)
            {
                find = this.myCollection[id];
            }
            else
            {
                find = new Dictionary<string, IvsAndEudc>();
            }
            ret = new ReadOnlyCollection<IvsAndEudc>(find.Values.ToList());
            return ret;
        }

        public ReadOnlyCollection<IvsAndEudc> this[Workbook wb]
        {
            get { return this.GetItem(Utility.GetID(wb)); }
        }

        public ReadOnlyCollection<IvsAndEudc> this[Document doc]
        {
            get { return this.GetItem(Utility.GetID(doc)); }
        }

        public ReadOnlyCollection<IvsAndEudc> this[Presentation ppt]
        {
            get { return this.GetItem(Utility.GetID(ppt)); }
        }

        private IvsAndEudc GetItem(Guid id, string ivs, string fontName)
        {
            IvsAndEudc ret = null;

            if (id == Guid.Empty)
            {
                throw new ArgumentException("idがGuid.Emptyです");
            }
            if (string.IsNullOrWhiteSpace(ivs))
            {
                throw new ArgumentNullException("ivsがnullまたは空です");
            }
            if (string.IsNullOrWhiteSpace(fontName))
            {
                throw new ArgumentNullException("ivsがnullまたは空です");
            }

            if (this.myCollection.ContainsKey(id) == true)
            {
                Dictionary<string, IvsAndEudc> find = this.myCollection[id];
                string key = IvsAndEudc.CreateKey(ivs, fontName);
                if (find.ContainsKey(key) == true)
                {
                    ret = find[key];
                }
            }

            return ret;
        }

        public IvsAndEudc this[Workbook wb, string ivs, string fontName]
        {
            get { return this.GetItem(Utility.GetID(wb), ivs, fontName); }
        }

        public IvsAndEudc this[Document doc, string ivs, string fontName]
        {
            get { return this.GetItem(Utility.GetID(doc), ivs, fontName); }
        }

        public IvsAndEudc this[Presentation ppt, string ivs, string fontName]
        {
            get { return this.GetItem(Utility.GetID(ppt), ivs, fontName); }
        }
    }
}
