﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Microsoft.Office.Core;
using Office = Microsoft.Office.Core;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;

namespace IvsAddinCommon
{
    public partial class PowerPointHelper : BaseHelperClass
    {
        private Dictionary<Guid, string> mPptNameAndRibbonControlID = new Dictionary<Guid, string>();

        private DataObject ClipboardCopyOrCut(CopyOrCut coc, PowerPoint.Application app, ref bool CancelDefault)
        {
            DataObject dataObj;
            CancelDefault = false;  //そのままPowerPointに任せる=false
            dataObj = null;

            PowerPoint.Selection selection = app.ActiveWindow.Selection;

            if ((selection != null)
                && (selection.Type == PowerPoint.PpSelectionType.ppSelectionText))
            {
                ReadOnlyCollection<IvsAndEudc> IvsAndEudcList = base.IvsAndEudcList[app.ActivePresentation];
                if (IvsAndEudcList != null && 0 < IvsAndEudcList.Count)
                {
                    switch (coc)
                    {
                        case CopyOrCut.Copy:
                            selection.Copy();
                            break;
                        case CopyOrCut.Cut:
                            selection.Cut();
                            break;
                        default:
                            throw new NotImplementedException();
                    }
                    Application.DoEvents();
                    using (RichTextBox rtb = new RichTextBox())
                    {
                        Application.DoEvents();
                        rtb.Paste();
                        Application.DoEvents();

                        dataObj = base.RichTextEucd2Ivs(IvsAndEudcList, rtb, ref CancelDefault);
                    }
                }
            }
            return dataObj;
        }

        /// <summary>
        /// コピー
        /// </summary>
        /// <param name="app"></param>
        /// <param name="CancelDefault"></param>
        /// <returns></returns>
        public DataObject ClipboardCopyFromPP(PowerPoint.Application app, ref bool CancelDefault)
        {
            DataObject dataObj = this.ClipboardCopyOrCut(CopyOrCut.Copy, app, ref CancelDefault);
            return dataObj;
        }

        /// <summary>
        /// カット
        /// </summary>
        /// <param name="app"></param>
        /// <param name="CancelDefault"></param>
        /// <returns></returns>
        public DataObject ClipboardCutFromPP(PowerPoint.Application app, ref bool CancelDefault)
        {
            DataObject dataObj = this.ClipboardCopyOrCut(CopyOrCut.Cut, app, ref CancelDefault);
            return dataObj;
        }

        /// <summary>
        /// ペースト
        /// </summary>
        /// <param name="app"></param>
        /// <param name="CancelDefault"></param>
        /// <returns></returns>
        public DataObject ClipboardPasteToPP(PowerPoint.Application app, ref bool CancelDefault)
        {
            DataObject dataObj = base.ClipboardPasteTo(app, ref CancelDefault);
            return dataObj;
        }

        public bool ExistIvs(PowerPoint.Presentation ppt)
        {
            bool result = false;
            int counter = 0;
            //string[] ivsSelectorList = Utility.IVList;
            string[] ivsSelectorList = new string[] { Utility.IVS_HIGH_SURROGATE_CHAR.ToString() };

            Application.UseWaitCursor = true;
            try
            {
                foreach (PowerPoint.Slide slide in ppt.Slides)
                {
                    foreach (PowerPoint.Shape shape in slide.Shapes)
                    {
                        counter += this.ShapeSearchOrReplace(ActionMode.IvsSearchOnly, ppt, shape, ivsSelectorList, Utility.IVS_HIGH_SURROGATE_CHAR);    //通常の文字たち

                        if (0 < counter)
                        {
                            break;
                        }
                    }

                    if (0 < counter)
                    {
                        break;
                    }
                }
            }
            finally
            {
                Application.UseWaitCursor = false;
            }

            if (0 < counter)
            {
                result = true;
            }
            else
            {
                result = false;
            }
            return result;
        }

        public int Ivs2Eudc(PowerPoint.Presentation ppt)
        {
            int intResult = 0;

            if (ppt != null)
            {
                Application.UseWaitCursor = true;
                MsoTriState savedFlag = ppt.Saved;
                try
                {
                    intResult = this.myIvs2Eudc(ppt);
                }
                catch (ReserveEudcNullResultException ex)
                {
                    Utility.ShowErrorMsgBox(ex);
                }
                finally
                {
                    ppt.Saved = savedFlag;
                    Application.UseWaitCursor = false;
                }
            }
            return intResult;
        }

        private int myIvs2Eudc(PowerPoint.Presentation ppt)
        {
            int counter = 0;
            string[] ivsSelectorList = Utility.IVList;

            base.IvsAndEudcList.Remove(ppt);

            foreach (PowerPoint.Slide slide in ppt.Slides)
            {
                foreach (PowerPoint.Shape shape in slide.Shapes)
                {
                    int IvsHighSurrogateCount = this.ShapeSearchOrReplace(ActionMode.IvsSearchOnly, ppt, shape, new string[] { Utility.IVS_HIGH_SURROGATE_CHAR.ToString() }, Utility.IVS_HIGH_SURROGATE_CHAR);
                    if (0 < IvsHighSurrogateCount)
                    {
                        counter += this.ShapeSearchOrReplace(ActionMode.Ivs2Eudc, ppt, shape, ivsSelectorList, Utility.IVS_HIGH_SURROGATE_CHAR);    //通常の文字たち
                    }
                }
            }
            return counter;
        }


        private enum ActionMode
        {
            IvsSearchOnly,
            Ivs2Eudc,
            Eudc2Ivs,
        }

        private int ShapeSearchOrReplace(ActionMode mode, PowerPoint.Presentation ppt, PowerPoint.Shape shape, string[] findWhatList, char IvsHighSurrogate)
        {
            int counter = 0;

            if ((mode == ActionMode.IvsSearchOnly) && (0 < counter))
            {
                //スルー
            }
            else
            {
                counter += this.mySearchOrReplace(mode, ppt, shape, findWhatList, IvsHighSurrogate);
            }


            if ((mode == ActionMode.IvsSearchOnly) && (0 < counter))
            {
                //スルー
            }
            else
            {
                counter += this.TableSearchOrReplace(mode, ppt, shape, findWhatList, IvsHighSurrogate);
            }

            if ((mode == ActionMode.IvsSearchOnly) && (0 < counter))
            {
                //スルー
            }
            else
            {
                counter += this.GroupSearchOrReplace(mode, ppt, shape, findWhatList, IvsHighSurrogate);
            }

            return counter;
        }

        private int mySearchOrReplace(ActionMode mode, PowerPoint.Presentation ppt, PowerPoint.Shape shape, string[] findWhatList, char IvsHighSurrogate)
        {
            int counter = 0;

            MsoTriState hasTextFrame;
            try
            {
                hasTextFrame = shape.HasTextFrame;
            }
            catch (COMException)
            {
                hasTextFrame = MsoTriState.msoFalse;    //2010 only
            }

            if (hasTextFrame == MsoTriState.msoTrue && shape.TextFrame.HasText == MsoTriState.msoTrue)
            {
                PowerPoint.TextRange textRange = shape.TextFrame.TextRange;

                if (mode == ActionMode.Ivs2Eudc || mode == ActionMode.IvsSearchOnly)
                {
                    PowerPoint.TextRange findIvsHighSurrogate = textRange.Find(FindWhat: IvsHighSurrogate.ToString());
                    if (findIvsHighSurrogate == null)
                    {
                        return counter;
                    }
                }

                foreach (string findWhat in findWhatList)
                {
                    if (mode == ActionMode.Ivs2Eudc || mode == ActionMode.IvsSearchOnly)
                    {
                        PowerPoint.TextRange findIvsHighSurrogate = textRange.Find(FindWhat: IvsHighSurrogate.ToString());
                        if (findIvsHighSurrogate == null)
                        {
                            break;
                        }
                    }

                    PowerPoint.TextRange findTextRange = textRange.Find(FindWhat: findWhat);
                    while (findTextRange != null)
                    {
                        switch (mode)
                        {
                            case ActionMode.IvsSearchOnly:
                                counter++;
                                return counter;
                            case ActionMode.Ivs2Eudc:
                                PowerPoint.TextRange ivsTextRange = textRange.Characters(Start: findTextRange.Start - 1, Length: findWhat.Length + 1);
                                string ivsText = ivsTextRange.Text;
                                counter += this.ReplaceIvs2Eudc(ppt, ivsTextRange); //置換
                                findTextRange = textRange.Find(FindWhat: findWhat);   //検索
                                break;
                            case ActionMode.Eudc2Ivs:
                                counter += this.ReplaceEudc2Ivs(ppt, findTextRange); //置換
                                findTextRange = textRange.Find(FindWhat: findWhat);   //検索
                                break;
                            default:
                                throw new NotImplementedException();
                        }
                    }
                }
            }
            return counter;
        }

        private int ReplaceEudc2Ivs(PowerPoint.Presentation ppt, PowerPoint.TextRange findTextRange)
        {
            int ret = 0;

            string eudc = findTextRange.Text;
            PowerPoint.TextRange ivsFirstChar = findTextRange.Characters(Start: 1, Length: 1);
            string eudcFontNameFarEast = ivsFirstChar.Font.NameFarEast;
            ReadOnlyCollection<IvsAndEudc> IvsAndEudcCollection = base.IvsAndEudcList[ppt];
            if (IvsAndEudcCollection != null && 0 < IvsAndEudcCollection.Count(x => x.Eudc == eudc && x.FontName == eudcFontNameFarEast))
            {
                IvsAndEudc ivsAndEudc = base.IvsAndEudcList[ppt].First(x => x.Eudc == eudc && x.FontName == eudcFontNameFarEast);

                findTextRange.Text = ivsAndEudc.Ivs;
                findTextRange.Font.NameFarEast = ivsAndEudc.FontName;
                ret = 1;
            }
            return ret;
        }

        private int ReplaceIvs2Eudc(PowerPoint.Presentation ppt, PowerPoint.TextRange ivsTextRange)
        {
            int ret = 0;
            string ivsText = ivsTextRange.Text;
            PowerPoint.TextRange ivsFirstChar = ivsTextRange.Characters(Start: 1, Length: 1);
            string ivsTextFontNameFarEast = ivsFirstChar.Font.NameFarEast;

            OfficeIVS.ICharMap charMap = OfficeIVS.FontManager.GetCharMap(ivsTextFontNameFarEast);
            string eudc = charMap.ReserveEUDC(ivsText);

            if (eudc == null)
            {
                throw new ReserveEudcNullResultException("ReplaceIvs2Eudc");
            }
            charMap.ApplyToFont();

            ivsTextRange.Text = eudc; //文字の差し替え
            ivsTextRange.Font.NameFarEast = ivsTextFontNameFarEast;   //フォントの変更

            if (base.IvsAndEudcList.Contains(ppt, ivsText, ivsTextFontNameFarEast) == false)
            {
                base.IvsAndEudcList.Add(ppt, ivsText, eudc, ivsTextFontNameFarEast);   //情報を保存
            }
            ret = 1;

            return ret;
        }

        private int GroupSearchOrReplace(ActionMode mode, PowerPoint.Presentation ppt, PowerPoint.Shape shape, string[] ivsSelectorList, char IvsHighSurrogate)
        {
            int counter = 0;

            try
            {
                if (shape.Type == MsoShapeType.msoGroup)
                {
                    foreach (PowerPoint.Shape childShape in shape.GroupItems)
                    {
                        counter += this.ShapeSearchOrReplace(mode, ppt, childShape, ivsSelectorList, IvsHighSurrogate);

                        if ((mode == ActionMode.IvsSearchOnly) && (0 < counter))
                        {
                            return counter;
                        }
                    }
                }
            }
            catch (NotImplementedException)
            {
                //http://www.add-in-express.com/forum/read.php?FID=5&TID=6959
            }
            catch (COMException)
            {
                //2010 only
            }

            return counter;
        }

        private int TableSearchOrReplace(ActionMode mode, PowerPoint.Presentation ppt, PowerPoint.Shape shape, string[] ivsSelectorList, char IvsHighSurrogate)
        {
            int counter = 0;

            MsoTriState hasTable;
            try
            {
                hasTable = shape.HasTable;
            }
            catch (COMException)
            {
                hasTable = MsoTriState.msoFalse;    //2010 only
            }

            if (hasTable == MsoTriState.msoTrue)   //テーブル内
            {
                foreach (PowerPoint.Row r in shape.Table.Rows)
                {
                    foreach (PowerPoint.Cell c in r.Cells)
                    {
                        if (c.Shape != null)
                        {
                            PowerPoint.Shape currentShape = c.Shape;
                            counter += this.ShapeSearchOrReplace(mode, ppt, currentShape, ivsSelectorList, IvsHighSurrogate);

                            if ((mode == ActionMode.IvsSearchOnly) && (0 < counter))
                            {
                                return counter;
                            }
                        }
                    }
                }
            }
            return counter;
        }

        public int Eudc2Ivs(PowerPoint.Presentation ppt)
        {
            int intResult = 0;

            if (ppt != null)
            {
                Application.UseWaitCursor = true;
                MsoTriState savedFlag = ppt.Saved;
                try
                {
                    intResult = this.myEudc2Ivs(ppt);
                }
                finally
                {
                    Application.UseWaitCursor = false;
                }
                ppt.Saved = savedFlag;
            }
            return intResult;
        }

        private int myEudc2Ivs(PowerPoint.Presentation ppt)
        {
            int counter = 0;
            ReadOnlyCollection<IvsAndEudc> IvsAndEudcCollectino = base.IvsAndEudcList[ppt];

            if (ppt != null && IvsAndEudcCollectino != null && 0 < IvsAndEudcCollectino.Count)
            {
                string[] eudcList = IvsAndEudcCollectino.Select(x => x.Eudc).ToArray();

                foreach (PowerPoint.Slide slide in ppt.Slides)
                {
                    foreach (PowerPoint.Shape shape in slide.Shapes)
                    {
                        counter += this.ShapeSearchOrReplace(ActionMode.Eudc2Ivs, ppt, shape, eudcList, Utility.IVS_HIGH_SURROGATE_CHAR);
                    }
                }
            }

            return counter;
        }

        internal bool ExistEudc(PowerPoint.Presentation ppt)
        {
            bool ret = true;
            if (0 < base.IvsAndEudcList[ppt].Count)
            {
                ret = true;
            }
            else
            {
                ret = false;
            }
            return ret;
        }

        public void PresentationOpen(PowerPoint.Presentation Pres)
        {
            bool ret = base.IvsAndEudcList.Remove(Pres);

            if (this.ExistIvs(Pres) == true)
            {
                this.Ivs2Eudc(Pres);
            }
        }

        public void PresentationBeforeSave(PowerPoint.Presentation Pres, ref bool Cancel)
        {
            if (this.ExistEudc(Pres) == true)
            {
                Guid presID = Utility.GetID(Pres);
                //bool SaveAsUI = !Pres.FullName.Contains(".");
                bool SaveAsUI = !Path.IsPathRooted(Pres.FullName);

                if (SaveAsUI == false)
                {
                    if (this.mPptNameAndRibbonControlID.ContainsKey(presID) == true)
                    {
                        string controlID = this.mPptNameAndRibbonControlID[presID];
                        if (controlID != "FileSave")
                        {
                            SaveAsUI = true;
                        }
                    }
                    else
                    {
                        SaveAsUI = true;
                    }
                }

                if (SaveAsUI == true)
                {
                    this.mSaveInfo = this.CallSaveAsDialog(Pres, this.mPptNameAndRibbonControlID);
                    Cancel = true;  //PowerPoint自体の処理はスルー
                }
                else
                {
                    this.mSaveInfo = new SaveInfo(SaveInfo.ResultType.Save, Pres, "", PowerPoint.PpSaveAsFileType.ppSaveAsDefault);
                    Cancel = true;  //PowerPoint自体の処理はスルー
                }
            }
            else
            {
                this.mSaveInfo = new SaveInfo(SaveInfo.ResultType.NotFoundEudc, Pres, "", PowerPoint.PpSaveAsFileType.ppSaveAsDefault);
                Cancel = true;
            }
        }

        private SaveInfo CallSaveAsDialog(PowerPoint.Presentation Pres, Dictionary<Guid, string> pptNameAndControlID)
        {
            SaveInfo saveInfoItem = null;
            int result;
            Guid pptID = Utility.GetID(Pres);

            int filterIndex = 1;
            if (pptNameAndControlID.ContainsKey(pptID) == true)
            {
                filterIndex = this.ControlID2FilterIndex(Pres, pptNameAndControlID);
                pptNameAndControlID.Remove(pptID);
            }

            //http://support.microsoft.com/kb/222701
            Office.FileDialog dialog = Pres.Application.get_FileDialog(MsoFileDialogType.msoFileDialogSaveAs);
            dialog.InitialFileName = Path.GetFileName(Pres.FullName);
            dialog.FilterIndex = filterIndex;

            result = dialog.Show();
            if (result != 0)
            {
                //dialog.Execute();   //PowerPointが保存処理をするはずだが、なぜか処理しない

                PowerPoint.PpSaveAsFileType fileType = this.FilterIndex2FileType(dialog);

                //http://social.msdn.microsoft.com/Forums/en-US/vsto/thread/435d1149-a601-4d12-8674-a9575c62d26a
                saveInfoItem = new SaveInfo(SaveInfo.ResultType.SaveAs, Pres, dialog.SelectedItems.Item(1), fileType);
            }
            else
            {
                saveInfoItem = new SaveInfo(SaveInfo.ResultType.Cancel, Pres, "", PowerPoint.PpSaveAsFileType.ppSaveAsDefault);
            }

            return saveInfoItem;
        }

        private class SaveInfo
        {
            internal enum ResultType
            {
                Save,
                SaveAs,
                NotFoundEudc,
                Cancel,
            }

            internal ResultType Result { get; private set; }
            internal PowerPoint.Presentation Pres { get; private set; }
            internal string FileFullName { get; private set; }
            internal PowerPoint.PpSaveAsFileType FileFormat { get; private set; }

            internal SaveInfo(ResultType resultType, PowerPoint.Presentation pres, string fileFullName, PowerPoint.PpSaveAsFileType fileFormat)
            {
                this.Result = resultType;
                this.Pres = pres;
                this.FileFullName = fileFullName;
                this.FileFormat = fileFormat;
            }
        }

        private SaveInfo mSaveInfo;

        public void SaveEvent(PowerPoint.Presentation ppt, IRibbonControl control, ref bool CancelDefault)
        {
            Guid pptID = Utility.GetID(ppt);
            this.mPptNameAndRibbonControlID[pptID] = control.Id;

            this.mSaveInfo = null;
            CancelDefault = true;
            ppt.Application.PresentationBeforeSave += new PowerPoint.EApplication_PresentationBeforeSaveEventHandler(PresentationBeforeSave);
            try
            {
                ppt.Save();
            }
            catch (System.Runtime.InteropServices.COMException ex)
            {
                Debug.WriteLine("Message:" + ex.Message);
                Debug.WriteLine("Source:" + ex.Source);
                Debug.WriteLine("StackTrace:" + ex.StackTrace);
            }
            finally
            {
                ppt.Application.PresentationBeforeSave -= new PowerPoint.EApplication_PresentationBeforeSaveEventHandler(PresentationBeforeSave);
            }

            if (this.mSaveInfo != null)
            {
                switch (this.mSaveInfo.Result)
                {
                    case SaveInfo.ResultType.Save:
                        if (this.ExistEudc(this.mSaveInfo.Pres) == true)
                        {
                            this.Eudc2Ivs(this.mSaveInfo.Pres);
                        }
                        this.mSaveInfo.Pres.Save();
                        if (this.ExistIvs(this.mSaveInfo.Pres) == true)
                        {
                            this.Ivs2Eudc(this.mSaveInfo.Pres);
                        }
                        break;
                    case SaveInfo.ResultType.SaveAs:
                        if (this.ExistEudc(this.mSaveInfo.Pres) == true)
                        {
                            this.Eudc2Ivs(this.mSaveInfo.Pres);
                        }
                        this.mSaveInfo.Pres.SaveAs(FileName: this.mSaveInfo.FileFullName, FileFormat: this.mSaveInfo.FileFormat);
                        if (this.ExistIvs(this.mSaveInfo.Pres) == true)
                        {
                            this.Ivs2Eudc(this.mSaveInfo.Pres);
                        }
                        break;
                    case SaveInfo.ResultType.NotFoundEudc:
                        CancelDefault = false;
                        break;
                    case SaveInfo.ResultType.Cancel:
                        break;
                    default:
                        throw new NotImplementedException();
                }
            }
            this.mPptNameAndRibbonControlID.Remove(pptID);
            this.mSaveInfo = null;
        }

        private int ControlID2FilterIndex(PowerPoint.Presentation ppt, Dictionary<Guid, string> pptNameAndControlID)
        {
            int filterIndex = 1;
            string controlID;
            Guid pptID = Utility.GetID(ppt);

            if (pptNameAndControlID.ContainsKey(pptID) == true)
            {
                controlID = pptNameAndControlID[pptID];
            }
            else
            {
                controlID = "";
            }

            switch (controlID)
            {
                case "FileSave":
                    filterIndex = 1;
                    break;
                case "FileSaveAs":
                    filterIndex = 1;
                    break;
                case "FileSaveAsPowerPointPptx":
                    filterIndex = 1;
                    break;
                case "FileSaveAsPowerPointPpsx":
                    filterIndex = 10;
                    break;
                case "FileSaveAsPowerPoint97_2003":
                    filterIndex = 3;
                    break;
                case "FileSaveAsPdfOrXps":
                    filterIndex = 4;
                    throw new NotImplementedException();
                case "FileSaveAsOtherFormats":
                    filterIndex = 1;
                    break;
                case "FileSaveAsPowerPointOpenDocumentPresentation":
                    filterIndex = 23;
                    break;
            }

            return filterIndex;
        }

        public void PresentationClose(PowerPoint.Presentation Pres)
        {
            //while (this.mTimerForm != null)
            //{
            //    Application.DoEvents();
            //    Thread.Sleep(100);
            //}

            if (Pres.Saved == MsoTriState.msoFalse && this.ExistEudc(Pres) == true)
            {
                MessageBox.Show("外字に割り当てられた異体字が含まれている可能性があります。", "アドイン", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        public void DecisionClicked(PowerPoint.Application app, string eudc, string fontName)
        {
            if (app.ActiveWindow != null)
            {
                PowerPoint.Selection selec = app.ActiveWindow.Selection;
                if (selec != null && selec.Type == PowerPoint.PpSelectionType.ppSelectionText)
                {
                    try
                    {
                        if (selec.TextRange != null)
                        {
                            PowerPoint.TextRange tr = selec.TextRange.InsertBefore(eudc);

                            string saveFontName = tr.Font.Name;
                            string saveFontNameAscii = tr.Font.NameAscii;
                            string saveNameComplexScript = tr.Font.NameComplexScript;
                            string saveFontNameFarEast = tr.Font.NameFarEast;
                            string saveFontNameOther = tr.Font.NameOther;

                            tr.Font.Name = fontName;
                            tr.Font.NameAscii = fontName;
                            tr.Font.NameComplexScript = fontName;
                            tr.Font.NameFarEast = fontName;
                            tr.Font.NameOther = fontName;

                            selec.TextRange.Font.Name = saveFontName;
                            selec.TextRange.Font.NameAscii = saveFontNameAscii;
                            selec.TextRange.Font.NameComplexScript = saveNameComplexScript;
                            selec.TextRange.Font.NameFarEast = saveFontNameFarEast;
                            selec.TextRange.Font.NameOther = saveFontNameOther;

                            PowerPoint.Presentation ppt = selec.Application.ActivePresentation;
                            OfficeIVS.ICharMap charMap = OfficeIVS.FontManager.GetCharMap(fontName);
                            string ivs = charMap.GetIvs(eudc);
                            base.IvsAndEudcList.Add(ppt, ivs, eudc, fontName);
                        }
                    }
                    catch (System.ArgumentException ex)
                    {
                        //SmartArtの場所によっては「このメンバーにアクセスできるのは、単一の図形の場合だけです。」が発生することがあります。
                        //Utility.ShowErrorMsgBox(ex);
                    }
                }
            }
        }

        public override void PasteRichText(object officeApp, IDataObject clipboardData, RichTextBox rtb, Dictionary<string, IvsAndEudc> IvsAndEudcDictionary, ref bool CancelDefault)
        {
            PowerPoint.Application appPP = officeApp as PowerPoint.Application;

            if (appPP != null && appPP.ActiveWindow != null && appPP.ActiveWindow.Selection != null)
            {
                DataObject saveData = base.CreateCopy(clipboardData);
                rtb.Select(0, rtb.TextLength);
                Application.DoEvents();
                DataObject newData = new DataObject(DataFormats.Rtf, rtb.SelectedRtf);
                Application.DoEvents();
                Clipboard.Clear();
                Clipboard.SetDataObject(newData);
                Application.DoEvents();
                appPP.ActiveWindow.Selection.TextRange.Paste();
                Application.DoEvents();
                Clipboard.SetDataObject(saveData);
                CancelDefault = true;

                foreach (string item in IvsAndEudcDictionary.Keys)
                {
                    IvsAndEudc iae = IvsAndEudcDictionary[item];
                    this.IvsAndEudcList.Add(appPP.ActivePresentation, iae.Ivs, iae.Eudc, iae.FontName);
                }
            }
        }
    }
}
