﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Microsoft.Office.Tools.Word;
using Office = Microsoft.Office.Core;
using Word = Microsoft.Office.Interop.Word;

namespace IvsAddinCommon
{
    public class WordHelper : BaseHelperClass
    {
        private ApplicationFactory mAppFactory = null;
        private Dictionary<Guid, int?> mDocIdAndSaveFormat = new Dictionary<Guid, int?>();

        public WordHelper()
        {
        }

        public bool HasApplicationFactory
        {
            get
            {
                return this.mAppFactory != null;
            }
        }

        public void SetApplicationFactory(ApplicationFactory appFactory)
        {
            this.mAppFactory = appFactory;
        }

        /// <summary>
        /// コピー
        /// </summary>
        /// <param name="app"></param>
        /// <param name="CancelDefault"></param>
        /// <returns></returns>
        public DataObject ClipboardCopyFromWord(Word.Application app, ref bool CancelDefault)
        {
            DataObject dataObj = this.ClipboardCopyOrCut(CopyOrCut.Copy, app, ref CancelDefault);
            return dataObj;
        }

        /// <summary>
        /// カット
        /// </summary>
        /// <param name="app"></param>
        /// <param name="CancelDefault"></param>
        /// <returns></returns>
        public DataObject ClipboardCutFromWord(Word.Application app, ref bool CancelDefault)
        {
            DataObject dataObj = this.ClipboardCopyOrCut(CopyOrCut.Cut, app, ref CancelDefault);
            return dataObj;
        }

        private DataObject ClipboardCopyOrCut(CopyOrCut coc, Word.Application app, ref bool CancelDefault)
        {
            CancelDefault = false;  //そのままExcelに任せる=false
            DataObject dataObj = null;

            try
            {
                if (app != null && app.Selection != null && app.ActiveDocument != null)
                {
                    Microsoft.Office.Tools.Word.Document myDoc = this.mAppFactory.GetVstoObject(app.ActiveDocument);
                    ReadOnlyCollection<IvsAndEudc> IvsAndEudcList = base.IvsAndEudcList[myDoc];
                    if (IvsAndEudcList != null && 0 < IvsAndEudcList.Count)
                    {
                        Application.DoEvents();
                        switch (coc)
                        {
                            case CopyOrCut.Copy:
                                app.Selection.Copy();
                                break;
                            case CopyOrCut.Cut:
                                app.Selection.Cut();
                                break;
                            default:
                                throw new NotImplementedException();
                        }
                        Application.DoEvents();
                        using (RichTextBox rtb = new RichTextBox())
                        {
                            Application.DoEvents();
                            rtb.Paste();
                            Application.DoEvents();

                            dataObj = base.RichTextEucd2Ivs(IvsAndEudcList, rtb, ref CancelDefault);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
                Debug.WriteLine(ex.StackTrace);
                CancelDefault = false;
            }

            return dataObj;
        }

        /// <summary>
        /// ペースト
        /// </summary>
        /// <param name="app"></param>
        /// <param name="CancelDefault"></param>
        /// <returns></returns>
        public DataObject ClipboardPasteToWord(Word.Application app, ref bool CancelDefault)
        {
            DataObject dataObj = base.ClipboardPasteTo(app, ref CancelDefault);
            return dataObj;
        }

        public int Ivs2Eudc(Document doc)
        {
            int intResult = 0;

            if (doc != null)
            {
                doc.Application.System.Cursor = Word.WdCursorType.wdCursorWait;
                Application.DoEvents();
#if !DEBUG
                doc.Application.ScreenUpdating = false;
#endif
                bool savedFlag = doc.Saved;
                try
                {
                    intResult = this.myIvs2Eudc(doc);
                }
                catch (ReserveEudcNullResultException ex)
                {
                    Utility.ShowErrorMsgBox(ex);
                }
                finally
                {
                    doc.Saved = savedFlag;
                    doc.Application.ScreenUpdating = true;
                    doc.Application.System.Cursor = Word.WdCursorType.wdCursorNormal;
                }
            }
            return intResult;
        }

        private int myIvs2Eudc(Document doc)
        {
            int counter = 0;
            string[] ivsSelectorList = Utility.IVList;

            base.IvsAndEudcList.Remove(doc);

            for (int i = 0; i < ivsSelectorList.Length; i++)
            {
                string currentIvsSelector = ivsSelectorList[i];
                Word.Find fnd = this.CreateFindObject(doc, currentIvsSelector);

                do
                {
                    bool result = fnd.Execute();
                    if (fnd.Found == true)
                    {
                        Word.Range findRange = (Word.Range)fnd.Parent;

                        //IVS全体に選択範囲を拡大
                        int moveResult = findRange.MoveStart(Unit: Word.WdUnits.wdCharacter, Count: -1);
                        string ivsText = findRange.Text;

                        //先頭文字を取得
                        Word.Range firstChar = doc.Range(findRange.Start, findRange.Start + 1);
                        string firstCharText = firstChar.Text;
                        string ivsTextFontName = firstChar.Font.NameFarEast;

                        OfficeIVS.ICharMap charMap = OfficeIVS.FontManager.GetCharMap(ivsTextFontName);
                        string eudc = charMap.ReserveEUDC(ivsText);
                        if (eudc == null)
                        {
                            throw new ReserveEudcNullResultException("myIvs2Eudc");
                        }
                        charMap.ApplyToFont();
                        base.IvsAndEudcList.Add(doc, ivsText, eudc, ivsTextFontName);

                        Word.Find fndReplace = this.CreateReplaceObject(doc, ivsText, ivsTextFontName, eudc, ivsTextFontName);
                        bool replaceResult = fndReplace.Execute(Replace: Word.WdReplace.wdReplaceAll);    //置換
                        if (replaceResult == false)
                        {
                            break;
                        }

                        counter++;
                    }

                } while (fnd.Found == true);

                if (this.ExistIvs(doc) == false)
                {
                    break;
                }
            }

            return counter;
        }

        private Word.Find CreateFindObject(Document document, string target)
        {
            Word.Range range = document.Content;
            Word.Find fnd = range.Find;

            fnd.ClearFormatting();    //検索条件のクリア
            fnd.Replacement.ClearFormatting();    //置換後の文字の設定条件クリア

            fnd.Text = target;

            fnd.Forward = true;
            fnd.Wrap = Word.WdFindWrap.wdFindContinue;
            fnd.Format = true;
            fnd.MatchCase = false;
            fnd.MatchWholeWord = false;
            fnd.MatchByte = false;
            fnd.MatchAllWordForms = false;
            fnd.MatchSoundsLike = false;
            fnd.MatchWildcards = false;
            fnd.MatchFuzzy = false;

            return fnd;
        }

        private Word.Find CreateReplaceObject(Document document, string target, string targetFontName, string newString, string newStringFontName)
        {
            Word.Range range = document.Content;
            Word.Find fnd = range.Find;

            fnd.ClearFormatting();    //検索条件のクリア
            fnd.Replacement.ClearFormatting();    //置換後の文字の設定条件クリア

            fnd.Text = target;
            try
            {
                fnd.Font.NameFarEast = targetFontName;
            }
            catch (COMException ex)
            {
                Utility.ShowErrorMsgBox(ex);
            }

            fnd.Replacement.Text = newString;
            fnd.Replacement.Font.Name = targetFontName;
            try
            {
                fnd.Replacement.Font.NameFarEast = newStringFontName;
            }
            catch (COMException ex)
            {
                Utility.ShowErrorMsgBox(ex);
            }

            fnd.Forward = true;
            fnd.Wrap = Word.WdFindWrap.wdFindContinue;
            fnd.Format = true;
            fnd.MatchCase = false;
            fnd.MatchWholeWord = false;
            fnd.MatchByte = false;
            fnd.MatchAllWordForms = false;
            fnd.MatchSoundsLike = false;
            fnd.MatchWildcards = false;
            fnd.MatchFuzzy = false;

            return fnd;
        }

        public int Eudc2Ivs(Document doc)
        {
            int intResult = 0;

            if (doc != null)
            {
                doc.Application.System.Cursor = Word.WdCursorType.wdCursorWait;
                Application.DoEvents();
#if !DEBUG
                doc.Application.ScreenUpdating = false;
#endif
                try
                {
                    bool savedFlag = doc.Saved;
                    intResult = this.myEudc2Ivs(doc);
                    doc.Saved = savedFlag;
                }
                finally
                {
                    doc.Application.ScreenUpdating = true;
                    doc.Application.System.Cursor = Word.WdCursorType.wdCursorNormal;
                }
            }
            return intResult;
        }

        private int myEudc2Ivs(Document doc)
        {
            int counter = 0;

            foreach (IvsAndEudc currentIvsAndEudc in base.IvsAndEudcList[doc])
            {
                Word.Find fndReplace = this.CreateReplaceObject(doc, currentIvsAndEudc.Eudc, currentIvsAndEudc.FontName, currentIvsAndEudc.Ivs, currentIvsAndEudc.FontName);
                bool replaceResult = fndReplace.Execute(Replace: Word.WdReplace.wdReplaceAll);
                if (replaceResult == true)
                {
                    counter++;
                }
            }

            return counter;
        }

        public int CallSaveAsDialog(Document doc)
        {
            int result;

            int? saveFormat = null;
            Guid docID = Utility.GetID(doc);
            if (this.mDocIdAndSaveFormat.ContainsKey(docID) == true)
            {
                saveFormat = this.mDocIdAndSaveFormat[docID];
                this.mDocIdAndSaveFormat.Remove(docID);
            }
            if (saveFormat.HasValue == false)
            {
                saveFormat = (int)Word.WdSaveFormat.wdFormatXMLDocument;
            }

            if (this.ExistEudc(doc) == true)
            {
                this.Eudc2Ivs(doc);
            }

            //Word.Dialog dialog = doc.Application.Dialogs[Word.WdWordDialog.wdDialogFileSaveAs];
            dynamic dialog = doc.Application.Dialogs[Word.WdWordDialog.wdDialogFileSaveAs];
            dialog.Name = doc.Name;
            dialog.Format = saveFormat.Value;

            doc.Application.System.Cursor = Word.WdCursorType.wdCursorWait;
            Application.DoEvents();
            try
            {
                result = dialog.Show();
            }
            finally
            {
                doc.Application.System.Cursor = Word.WdCursorType.wdCursorNormal;
            }

            //dialog.Execute    //Execute呼ばずとも処理してくれる

            if (this.ExistIvs(doc) == true)
            {
                this.Ivs2Eudc(doc);
            }

            return result;
        }

        public bool ExistIvs(Document doc)
        {
            bool result = false;
            string[] ivsSelectorList = Utility.IVList;

            doc.Application.System.Cursor = Word.WdCursorType.wdCursorWait;
            Application.DoEvents();

            try
            {
                //Word.Find fndHighSurrogate = this.CreateFindObject(doc, Utility.IVS_HIGH_SURROGATE_CHAR.ToString());
                //fndHighSurrogate.MatchWildcards = true;              
                //bool findResult = fndHighSurrogate.Execute();
                //findResult = fndHighSurrogate.Found;

                //string regex = "[" + Utility.IVS_HIGH_SURROGATE_CHAR.ToString() + "]";
                //Word.Find fndSurrogate = this.CreateFindObject(doc, regex);
                //fndSurrogate.MatchWildcards = true;
                //bool findResult = fndSurrogate.Execute();
                //findResult = fndSurrogate.Found;

                //string regex = "[" + Utility.IVList[0] + "-" + Utility.IVList[Utility.IVList.Length - 1] + "]";
                //Word.Find fndSurrogate = this.CreateFindObject(doc, regex);
                //fndSurrogate.MatchWildcards = true;
                //bool findResult = fndSurrogate.Execute();
                //findResult = fndSurrogate.Found;

                //for (int i = 0; i < ivsSelectorList.Length; i++)
                //{
                //    Word.Find fnd = this.CreateFindObject(doc, ivsSelectorList[i]);
                //    bool ret = fnd.Execute();
                //    if (fnd.Found == true)
                //    {
                //        result = true;
                //        break;
                //    }
                //}

                result = doc.Content.Text.Contains(Utility.IVS_HIGH_SURROGATE_CHAR.ToString());
            }
            finally
            {
                doc.Application.System.Cursor = Word.WdCursorType.wdCursorNormal;
            }

            return result;
        }

        private bool ExistEudc(Document doc)
        {
            bool ret = true;

            doc.Application.System.Cursor = Word.WdCursorType.wdCursorWait;
            Application.DoEvents();
            try
            {
                if (0 < base.IvsAndEudcList[doc].Count)
                {
                    ret = true;
                }
                else
                {
                    ret = false;
                }
            }
            finally
            {
                doc.Application.System.Cursor = Word.WdCursorType.wdCursorNormal;
            }

            return ret;
        }

        public void SaveEvent(Document doc, Office.IRibbonControl control, ref bool CancelDefault)
        {
            //Word.WdSaveFormat? saveFormat = null;
            int? saveFormat = null;

            switch (control.Id)
            {
                case "FileSave":
                    saveFormat = (int)Word.WdSaveFormat.wdFormatXMLDocument;
                    break;
                case "FileSaveAs":
                    saveFormat = (int)Word.WdSaveFormat.wdFormatXMLDocument;
                    break;
                case "FileSaveAsWordDocx":
                    saveFormat = (int)Word.WdSaveFormat.wdFormatXMLDocument;
                    break;
                case "FileSaveAsWordDotx":
                    saveFormat = (int)Word.WdSaveFormat.wdFormatXMLTemplate;
                    break;
                case "FileSaveAsWord97_2003":
                    saveFormat = (int)Word.WdSaveFormat.wdFormatDocument97;
                    break;
                case "FileSaveAsPdfOrXps":
                    saveFormat = (int)Word.WdSaveFormat.wdFormatPDF;
                    throw new NotImplementedException();
                //break;
                case "FileSaveAsOtherFormats":
                    saveFormat = (int)Word.WdSaveFormat.wdFormatXMLDocument;
                    break;
                case "FileSaveAsWordOpenDocumentText":
                    //saveFormat = Word.WdSaveFormat.wdFormatOpenDocumentText;
                    saveFormat = 23;
                    break;
            }

            Guid docID = Utility.GetID(doc);
            this.mDocIdAndSaveFormat[docID] = saveFormat;

            CancelDefault = false;
        }

        private void SaveEx(Document doc)
        {
            doc.Application.System.Cursor = Word.WdCursorType.wdCursorWait;
            try
            {
                if (this.ExistEudc(doc) == true)
                {
                    this.Eudc2Ivs(doc);
                }

                doc.Save();

                if (this.ExistIvs(doc) == true)
                {
                    this.Ivs2Eudc(doc);
                }
            }
            finally
            {
                doc.Application.System.Cursor = Word.WdCursorType.wdCursorNormal;
            }
        }

        public void DocumentOpen(Word.Document Doc)
        {
            Document myDocument = this.mAppFactory.GetVstoObject(Doc);

            base.IvsAndEudcList.Remove(myDocument);

            if (this.ExistIvs(myDocument) == true)
            {
                this.Ivs2Eudc(myDocument);
                myDocument.UndoClear();
            }
        }

        public void DocumentBeforeSave(Word.Document Doc, ref bool SaveAsUI, ref bool Cancel)
        {
            Document myDocument = this.mAppFactory.GetVstoObject(Doc);

            if (SaveAsUI == true)
            {
                this.CallSaveAsDialog(myDocument);
                Cancel = true;  //Word自体の処理はスルー
            }
            else
            {
                if (string.IsNullOrWhiteSpace(myDocument.Path) == false)
                {
                    this.SaveEx(myDocument);
                    Cancel = true;  //Word自体の処理はスルー
                }
                else
                {
                    //Wordの自動保存
                }
            }
        }

        public void DecisionClicked(Word.Application app, string eudc, string fontName)
        {
            if (app.Selection != null
                && ((app.Selection.Type == Word.WdSelectionType.wdSelectionIP)
                   || (app.Selection.Type == Word.WdSelectionType.wdSelectionNormal)))
            {
                Word.Selection selc = app.Selection;

                string saveFontName = selc.Font.Name;
                string saveFontNameAscii = selc.Font.NameAscii;
                string saveFontNameBi = selc.Font.NameBi;
                string saveFontNameFarEast = selc.Font.NameFarEast;
                string saveFontNameOther = selc.Font.NameOther;

                selc.Font.Name = fontName;
                selc.Font.NameAscii = fontName;
                selc.Font.NameBi = fontName;
                selc.Font.NameFarEast = fontName;
                selc.Font.NameOther = fontName;

                selc.TypeText(eudc);

                selc.Font.Name = saveFontName;
                selc.Font.NameAscii = saveFontNameAscii;
                selc.Font.NameBi = saveFontNameBi;
                selc.Font.NameFarEast = saveFontNameFarEast;
                selc.Font.NameOther = saveFontNameOther;

                Document doc = this.mAppFactory.GetVstoObject(app.Selection.Document);
                OfficeIVS.ICharMap charMap = OfficeIVS.FontManager.GetCharMap(fontName);
                string ivs = charMap.GetIvs(eudc);
                base.IvsAndEudcList.Add(doc, ivs, eudc, fontName);
            }
        }

        public override void PasteRichText(object officeApp, IDataObject clipboardData, RichTextBox rtb, Dictionary<string, IvsAndEudc> IvsAndEudcDictionary, ref bool CancelDefault)
        {
            Word.Application appWord = officeApp as Word.Application;

            if (appWord != null && appWord.Selection != null && appWord.Selection.Range != null && appWord.Selection.Document != null)
            {
                DataObject saveData = base.CreateCopy(clipboardData);
                DataObject ret = new DataObject();
                rtb.Select(0, rtb.TextLength);
                Application.DoEvents();
                ret.SetData(DataFormats.Rtf, true, rtb.SelectedRtf);
                Application.DoEvents();
                Clipboard.Clear();
                Application.DoEvents();
                Clipboard.SetDataObject(ret);
                Application.DoEvents();
                try
                {
                    appWord.Selection.PasteAndFormat(Word.WdRecoveryType.wdPasteDefault);
                    CancelDefault = true;
                }
                catch (COMException comEx)
                {
                    Utility.ShowErrorMsgBox(comEx);
                    try
                    {
                        appWord.Selection.Paste();
                        CancelDefault = true;
                    }
                    catch (Exception ex)
                    {
                        Utility.ShowErrorMsgBox(ex);
                    }
                }
                finally
                {
                    Application.DoEvents();
                    Clipboard.SetDataObject(saveData);
                    Application.DoEvents();
                }

                foreach (string item in IvsAndEudcDictionary.Keys)
                {
                    IvsAndEudc iae = IvsAndEudcDictionary[item];
                    Document doc = this.mAppFactory.GetVstoObject(appWord.Selection.Document);
                    this.IvsAndEudcList.Add(doc, iae.Ivs, iae.Eudc, iae.FontName);
                }
            }
        }
    }
}
