﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml.Linq;
using Microsoft.Office.Tools.Word;
using Office = Microsoft.Office.Core;
using Word = Microsoft.Office.Interop.Word;
using System.Runtime.InteropServices;
using System.Windows.Forms.Integration;  

namespace Word2007IvsAddin
{
    public partial class ThisAddIn
    {
        private void ThisAddIn_Startup(object sender, System.EventArgs e)
        {
            OfficeIVS.FontManager.Initialize();
            this.Application.DocumentOpen += new Word.ApplicationEvents4_DocumentOpenEventHandler(Application_DocumentOpen);
            this.Application.DocumentBeforeSave += new Word.ApplicationEvents4_DocumentBeforeSaveEventHandler(Application_DocumentBeforeSave);
        }

        private void ThisAddIn_Shutdown(object sender, System.EventArgs e)
        {
            OfficeIVS.FontManager.Uninitialize();
        }

        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddIn_Startup);
            this.Shutdown += new System.EventHandler(ThisAddIn_Shutdown);
        }
        
        #endregion


        private AssistToolDialog.AssistControl mIvsInputForm = null;
        private MainRibbon ribbon;
        private IvsAddinCommon.WordHelper mWordHelper = new IvsAddinCommon.WordHelper();
        private UserControl userControl;
        private Microsoft.Office.Tools.CustomTaskPane customTaskPane;

        internal IvsAddinCommon.WordHelper WordHelper
        {
            get
            {
                if (this.mWordHelper.HasApplicationFactory == false)
                {
                    this.mWordHelper.SetApplicationFactory(Globals.Factory);
                }
                return this.mWordHelper;
            }
        }

        /// <summary>
        /// ドキュメントを開いたときのイベント
        /// </summary>
        /// <param name="Doc"></param>
        void Application_DocumentOpen(Word.Document Doc)
        {
            this.WordHelper.DocumentOpen(Doc);
        }
 
        /// <summary>
        /// ドキュメントを保存する直前のイベント
        /// </summary>
        /// <param name="Doc"></param>
        /// <param name="SaveAsUI"></param>
        /// <param name="Cancel"></param>
        void Application_DocumentBeforeSave(Word.Document Doc, ref bool SaveAsUI, ref bool Cancel)
        {
            this.WordHelper.DocumentBeforeSave(Doc, ref SaveAsUI, ref Cancel);
        }

        protected override object RequestService(Guid serviceGuid)
        {
            if (serviceGuid == typeof(Office.IRibbonExtensibility).GUID)
            {
                if (ribbon == null)
                    ribbon = new MainRibbon();
                return ribbon;
            }

            return base.RequestService(serviceGuid);
        }

        protected override Microsoft.Office.Core.IRibbonExtensibility CreateRibbonExtensibilityObject()
        {
            return new MainRibbon();
        }

        internal void IvsInputFormButtonClick()
        {
            if (this.mIvsInputForm == null)
            {
                mIvsInputForm = new AssistToolDialog.AssistControl(false);
                userControl = new UserControl();
                ElementHost eh = new ElementHost { Child = mIvsInputForm };
                userControl.Controls.Add(eh);
                //eh.Dock = DockStyle.Fill;
                // 枠にはまるようにサイズ調整
                eh.Height = 305;
                eh.Width = 300;
                customTaskPane = CustomTaskPanes.Add(userControl, "異体字入力");
                customTaskPane.DockPosition = Office.MsoCTPDockPosition.msoCTPDockPositionLeft;
                customTaskPane.Width = 305;
                customTaskPane.Visible = true;

                this.mIvsInputForm.DecisionClicked += new EventHandler(mIvsInputForm_DecisionClicked);
                this.customTaskPane.VisibleChanged += new EventHandler(customTaskPane_VisibleChanged);
            }
            else
            {
                customTaskPane.Visible = false;
            }
        }

        void customTaskPane_VisibleChanged(object sender, EventArgs e)
        {
            if (!customTaskPane.Visible)
                this.mIvsInputForm = null;
        }

        void mIvsInputForm_DecisionClicked(object sender, EventArgs e)
        {
            AssistToolDialog.AssistControl frm = sender as AssistToolDialog.AssistControl;
            if ((frm != null) && (string.IsNullOrWhiteSpace(frm.Eudc) == false))
            {
                if (Application.ActiveDocument != null)
                {
                    this.WordHelper.DecisionClicked(Application, frm.Eudc, frm.FontName);
                }
            }
        }
    }
}
