﻿using System;
using System.Windows.Forms;
using System.Windows.Forms.Integration;
using Excel = Microsoft.Office.Interop.Excel;
using Office = Microsoft.Office.Core;

namespace Excel2010IvsAddin
{
    public partial class ThisAddIn
    {
        private void ThisAddIn_Startup(object sender, System.EventArgs e)
        {
            OfficeIVS.FontManager.Initialize();
            this.Application.WorkbookOpen += new Excel.AppEvents_WorkbookOpenEventHandler(Application_WorkbookOpen);
            this.Application.WorkbookBeforeSave += new Excel.AppEvents_WorkbookBeforeSaveEventHandler(Application_WorkbookBeforeSave);
            this.Application.WorkbookAfterSave += new Excel.AppEvents_WorkbookAfterSaveEventHandler(Application_WorkbookAfterSave);
            this.Application.WorkbookBeforeClose += new Excel.AppEvents_WorkbookBeforeCloseEventHandler(Application_WorkbookBeforeClose);
            this.Application.SheetSelectionChange += new Excel.AppEvents_SheetSelectionChangeEventHandler(Application_SheetSelectionChange);
            this.Application.WorkbookActivate += new Excel.AppEvents_WorkbookActivateEventHandler(Application_WorkbookActivate);
            this.Application.SheetDeactivate += new Excel.AppEvents_SheetDeactivateEventHandler(Application_SheetDeactivate);
        }

        private void ThisAddIn_Shutdown(object sender, System.EventArgs e)
        {
            OfficeIVS.FontManager.Uninitialize();
        }

        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddIn_Startup);
            this.Shutdown += new System.EventHandler(ThisAddIn_Shutdown);
        }

        #endregion



        private AssistToolDialog.AssistControl mIvsInputForm = null;
        private MainRibbon ribbon;
        private IvsAddinCommon.Excel2010Helper mExcelHelper = new IvsAddinCommon.Excel2010Helper();
        private UserControl userControl;
        private Microsoft.Office.Tools.CustomTaskPane customTaskPane;

        internal IvsAddinCommon.Excel2010Helper ExcelHelper
        {
            get
            {
                if (this.mExcelHelper.HasApplicationFactory == false)
                {
                    this.mExcelHelper.SetApplicationFactory(Globals.Factory);
                }
                return this.mExcelHelper;
            }
        }

        void Application_WorkbookOpen(Excel.Workbook Wb)
        {
            this.ExcelHelper.WorkbookOpen(Wb);
        }

        void Application_WorkbookBeforeClose(Excel.Workbook Wb, ref bool Cancel)
        {
            this.ExcelHelper.WorkbookBeforeClose(Wb, ref Cancel);
        }

        void Application_WorkbookBeforeSave(Excel.Workbook Wb, bool SaveAsUI, ref bool Cancel)
        {
            this.ExcelHelper.WorkbookBeforeSave(Wb, SaveAsUI, ref Cancel);
        }

        void Application_WorkbookAfterSave(Excel.Workbook Wb, bool Success)
        {
            this.ExcelHelper.WorkbookAfterSave(Wb, Success);
        }

        /// <summary>
        /// セル変更イベント
        /// </summary>
        /// <param name="Sh"></param>
        /// <param name="Target"></param>
        void Application_SheetSelectionChange(object Sh, Excel.Range Target)
        {
            if (mIvsInputForm != null && customTaskPane != null && customTaskPane.Visible == true)
            {
                Application.Cursor = Excel.XlMousePointer.xlWait;
                System.Windows.Forms.Application.DoEvents();
                try
                {
                    mIvsInputForm.CellText = this.ExcelHelper.Range2ExStringList((Excel.Range)Target.get_Item(1));
                }
                finally
                {
                    Application.Cursor = Excel.XlMousePointer.xlDefault;
                }
            }
        }

        /// <summary>
        /// ブックがアクティブになったときに発生します。
        /// </summary>
        /// <param name="Wb"></param>
        void Application_WorkbookActivate(Excel.Workbook Wb)
        {
            this.SetCellText();
        }

        /// <summary>
        /// シート変更したとき
        /// </summary>
        /// <param name="Sh"></param>
        void Application_SheetDeactivate(object Sh)
        {
            SetCellText();
        }

        protected override object RequestService(Guid serviceGuid)
        {
            if (serviceGuid == typeof(Office.IRibbonExtensibility).GUID)
            {
                if (ribbon == null)
                    ribbon = new MainRibbon();
                return ribbon;
            }

            return base.RequestService(serviceGuid);
        }

        protected override Microsoft.Office.Core.IRibbonExtensibility CreateRibbonExtensibilityObject()
        {
            return new MainRibbon();
        }

        internal void IvsInputFormButtonClick()
        {
            if (this.mIvsInputForm == null)
            {
                mIvsInputForm = new AssistToolDialog.AssistControl(true);
                userControl = new UserControl();
                ElementHost eh = new ElementHost { Child = mIvsInputForm };
                userControl.Controls.Add(eh);
                //eh.Dock = DockStyle.Fill;
                // 枠にはまるようにサイズ調整
                eh.Height = 355;
                eh.Width = 300;
                customTaskPane = CustomTaskPanes.Add(userControl, "異体字入力");
                customTaskPane.DockPosition = Office.MsoCTPDockPosition.msoCTPDockPositionLeft;
                customTaskPane.Width = 305;
                customTaskPane.Visible = true;

                this.mIvsInputForm.DecisionClicked += new EventHandler(mIvsInputForm_DecisionClicked);

                this.SetCellText();
            }
            else
            {
                // 閉じる際にも保持する
                customTaskPane.Visible = !this.customTaskPane.Visible;  //Visibleを反転
                if (this.customTaskPane.Visible == false)
                {
                    this.CustomTaskPanes.Remove(this.customTaskPane);
                    this.mIvsInputForm = null;
                    this.userControl.Dispose();
                    this.userControl = null;
                    this.customTaskPane.Dispose();
                    this.customTaskPane = null;
                }
                else
                {
                    this.SetCellText();
                }
            }
        }

        void mIvsInputForm_DecisionClicked(object sender, EventArgs e)
        {
            AssistToolDialog.AssistControl frm = sender as AssistToolDialog.AssistControl;
            if ((frm != null) && (string.IsNullOrWhiteSpace(frm.Eudc) == false))
            {
                if (Application.ActiveWorkbook != null)
                {
                    this.ExcelHelper.DecisionClicked(Application, frm.Eudc, frm.FontName, frm.CaretIndex);

                    frm.SetCellText();
                }
            }
        }

        /// <summary>
        /// セルのテキストを入力フォームにセットする
        /// </summary>
        private void SetCellText()
        {
            if (this.Application.ActiveCell != null
                && this.mIvsInputForm != null
                && this.customTaskPane != null
                && this.customTaskPane.Visible == true)
            {
                Application.Cursor = Excel.XlMousePointer.xlWait;
                System.Windows.Forms.Application.DoEvents();
                try
                {
                    this.mIvsInputForm.CellText = this.ExcelHelper.Range2ExStringList(this.Application.ActiveCell);
                }
                finally
                {
                    Application.Cursor = Excel.XlMousePointer.xlDefault;
                }
            }
        }
    }
}