﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.Office.Core;
using Microsoft.Office.Interop.PowerPoint;
using Microsoft.Office.Tools.Excel;
using Microsoft.Office.Tools.Word;

namespace IvsAddinCommon
{
    public class Utility
    {
        //public const string IVS_HIGH_SURROGATE_STR = "\uDB40";
        public const char IVS_HIGH_SURROGATE_CHAR = '\uDB40';
        private static string[] mIVList = null;
        private const string PPT_TAGS_KYE_NAME = "Tag";
        public const string APP_NAME = "IVS対応アドイン";

        static Utility()
        {
            mIVList = GetIVList();
        }

        static private string[] GetIVList()
        {
            const string START = "\U000E0100";
            const string END = "\U000E01EF";

            int from = char.ConvertToUtf32(START, 0);
            int to = char.ConvertToUtf32(END, 0);
            List<string> sList = new List<string>();
            for (int i = from; i <= to; i++)
            {
                sList.Add(char.ConvertFromUtf32(i));
            }

            return sList.ToArray();
        }

        internal static string[] IVList
        {
            get
            {
                return mIVList;
            }
        }

        public static Guid GetID(Document doc)
        {
            Guid id = Guid.Empty;
            if (doc.Tag == null)
            {
                id = Guid.NewGuid();
                doc.Tag = id.ToString();
            }
            else
            {
                id = Guid.Parse(doc.Tag.ToString());
            }
            return id;
        }

        public static Guid GetID(Workbook wb)
        {
            Guid id = Guid.Empty;
            if (wb.Tag == null)
            {
                id = Guid.NewGuid();
                wb.Tag = id.ToString();
            }
            else
            {
                id = Guid.Parse(wb.Tag.ToString());
            }
            return id;
        }

        public static Guid GetID(Presentation ppt)
        {
            Guid id = Guid.Empty;
            if (string.IsNullOrEmpty(ppt.Tags[PPT_TAGS_KYE_NAME]) == true)
            {
                MsoTriState state = ppt.Saved;
                id = Guid.NewGuid();
                ppt.Tags.Add(PPT_TAGS_KYE_NAME, id.ToString());
                ppt.Saved = state;
            }
            else
            {
                id = Guid.Parse(ppt.Tags[PPT_TAGS_KYE_NAME]);
            }
            return id;
        }

        internal static void ShowErrorMsgBox(Exception ex)
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendLine(ex.GetType().FullName);

            sb.AppendLine("");
            sb.AppendLine("Message:");
            sb.AppendLine(ex.Message);

#if DEBUG
            ReserveEudcNullResultException renre = ex as ReserveEudcNullResultException;
            if (renre != null)
            {
                sb.AppendLine("");
                sb.AppendLine("MethodName:");
                sb.AppendLine(renre.MethodName);
            }

            sb.AppendLine("");
            sb.AppendLine("StackTrace:");
            sb.AppendLine(ex.StackTrace);
#endif

            MessageBox.Show(sb.ToString(), APP_NAME, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }

    public class ReserveEudcNullResultException : Exception
    {
        private static readonly string DEF_ERR_MSG = "EUDCに変換できない異体字が見つかりました。" + Environment.NewLine + "変換処理を中止します。";
        public string MethodName { get; private set; }

        public ReserveEudcNullResultException(string methodName)
            : base(DEF_ERR_MSG)
        {
            this.MethodName = methodName;
        }
    }
}
