﻿using System;
using Office = Microsoft.Office.Core;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;
using System.Windows.Forms;
using System.Windows.Forms.Integration;

namespace PowerPoint2010IvsAddin
{
    public partial class ThisAddIn
    {
        private void ThisAddIn_Startup(object sender, System.EventArgs e)
        {
            OfficeIVS.FontManager.Initialize();
            this.Application.PresentationOpen += new PowerPoint.EApplication_PresentationOpenEventHandler(Application_PresentationOpen);
            //this.Application.PresentationBeforeSave += new PowerPoint.EApplication_PresentationBeforeSaveEventHandler(Application_PresentationBeforeSave);
            this.Application.PresentationClose += new PowerPoint.EApplication_PresentationCloseEventHandler(Application_PresentationClose);
        }

        private void ThisAddIn_Shutdown(object sender, System.EventArgs e)
        {
            OfficeIVS.FontManager.Uninitialize();
        }

        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddIn_Startup);
            this.Shutdown += new System.EventHandler(ThisAddIn_Shutdown);
        }

        #endregion

        private AssistToolDialog.AssistControl mIvsInputForm = null;
        private MainRibbon ribbon;
        private readonly IvsAddinCommon.PowerPointHelper mPowerPoint = new IvsAddinCommon.PowerPointHelper();
        private UserControl userControl;
        private Microsoft.Office.Tools.CustomTaskPane customTaskPane;

        internal IvsAddinCommon.PowerPointHelper PowerPointHelper
        {
            get
            {
                return this.mPowerPoint;
            }
        }

        /// <summary>
        /// ファイルを開いたとき
        /// </summary>
        /// <param name="Pres"></param>
        void Application_PresentationOpen(PowerPoint.Presentation Pres)
        {
            this.PowerPointHelper.PresentationOpen(Pres);
        }

        /// <summary>
        /// ファイルを保存する直前
        /// </summary>
        /// <param name="Pres"></param>
        /// <param name="Cancel"></param>
        void Application_PresentationBeforeSave(PowerPoint.Presentation Pres, ref bool Cancel)
        {
            this.PowerPointHelper.PresentationBeforeSave(Pres, ref Cancel);
        }

        void Application_PresentationClose(PowerPoint.Presentation Pres)
        {
            this.PowerPointHelper.PresentationClose(Pres);
        }

        protected override object RequestService(Guid serviceGuid)
        {
            if (serviceGuid == typeof(Office.IRibbonExtensibility).GUID)
            {
                if (ribbon == null)
                    ribbon = new MainRibbon();
                return ribbon;
            }

            return base.RequestService(serviceGuid);
        }

        protected override Microsoft.Office.Core.IRibbonExtensibility CreateRibbonExtensibilityObject()
        {
            return new MainRibbon();
        }

        internal void IvsInputFormButtonClick()
        {
            if (this.mIvsInputForm == null)
            {
                mIvsInputForm = new AssistToolDialog.AssistControl(false);
                userControl = new UserControl();
                ElementHost eh = new ElementHost { Child = mIvsInputForm };
                userControl.Controls.Add(eh);
                //eh.Dock = DockStyle.Fill;
                // 枠にはまるようにサイズ調整
                eh.Height = 305;
                eh.Width = 300;
                customTaskPane = CustomTaskPanes.Add(userControl, "異体字入力");
                customTaskPane.DockPosition = Office.MsoCTPDockPosition.msoCTPDockPositionLeft;
                customTaskPane.Width = 305;
                customTaskPane.Visible = true;

                this.mIvsInputForm.DecisionClicked += new EventHandler(mIvsInputForm_DecisionClicked);
                this.customTaskPane.VisibleChanged += new EventHandler(customTaskPane_VisibleChanged);
            }
            else
            {
                customTaskPane.Visible = false;
            }
        }

        void customTaskPane_VisibleChanged(object sender, EventArgs e)
        {
            if (!customTaskPane.Visible)
                this.mIvsInputForm = null;
        }

        void mIvsInputForm_DecisionClicked(object sender, EventArgs e)
        {
            AssistToolDialog.AssistControl frm = sender as AssistToolDialog.AssistControl;
            if ((frm != null) && (string.IsNullOrWhiteSpace(frm.Eudc) == false))
            {
                if (Application.ActivePresentation != null)
                {
                    this.PowerPointHelper.DecisionClicked(Application, frm.Eudc, frm.FontName);
                }
            }
        }
    }
}
